Package: rfacebookstat
Type: Package
Title: Load Data from Facebook API Marketing
Version: 2.16.1
Authors@R: 
    c(person(given  = "Alexey",
             family = "Seleznev",
             role   = c("aut", "cre"),
             email  = "selesnow@gmail.com",
             comment = c(ORCID = "0000-0003-0410-7385")))
Maintainer: Alexey Seleznev <selesnow@gmail.com>
Description: Load data by campaigns, ads, ad sets and insights, ad account and business manager 
    from Facebook Marketing API into R. For more details see official documents by Facebook 
    Marketing API <https://developers.facebook.com/documentation/ads-commerce/marketing-api>.
Depends: R (>= 4.1.0)
BugReports: https://github.com/selesnow/rfacebookstat/issues
Imports: httr, httr2, jsonlite, dplyr, purrr, tidyr (>= 1.0.0),
        stringr, tidyselect, pbapply
URL: https://selesnow.github.io/rfacebookstat/,
        https://www.youtube.com/playlist?list=PLD2LDq8edf4pItOb-vZTG5AXZK2niJ8_R
License: MIT + file LICENSE
Suggests: knitr, rmarkdown
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-08-25 10:47:54 UTC; User
Author: Alexey Seleznev [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0410-7385>)
Repository: CRAN
Date/Publication: 2026-08-25 11:10:02 UTC
Built: R 4.5.2; ; 2026-08-25 13:01:14 UTC; unix

export(fbAuth)
export(fbGetToken)
export(fbGetBusinessManagers)
export(fbGetAdAccounts)
export(fbGetUserAdAccounts)
export(fbGetAdAccountUsers)
export(fbUpdateAdAccountUsers)
export(fbGetAdAccountsConversions)
export(fbDeleteAdAccountUsers)
export(fbGetPages)
export(fbGetApps)
export(fbGetAdAccountUsersPermissions)
export(fbGetLongTimeToken)
export(fbGetMarketingStat)
export(fbGetAdSets)
export(fbGetAdCreative)
export(fbGetAds)
export(fbGetCampaigns)
export(fbGetAdVideos)
export(fbGetBusinessUserAdAccounts)
export(fbGetBusinessManagersUsers)
export(fbGetCatalogs)
export(fbGetSettings)
export(fbGetCostData)
export(fbGetLogins)
export(fbSetUsername)
export(fbSetAccount)
export(fbSetBusinessId)
export(fbSetTokenPath)
export(fbSetApiVersion)
export(fbGetAdAccountCustomAudiences)

importFrom(jsonlite, fromJSON)
importFrom(jsonlite, toJSON)
importFrom(jsonlite, validate)
importFrom(httr, GET)
importFrom(httr, POST)
importFrom(httr, DELETE)
importFrom(httr, content)
importFrom(httr, headers)
importFrom(httr, stop_for_status)
importFrom(httr, status_code)
importFrom(dplyr, bind_rows)
importFrom(dplyr, bind_cols)
importFrom(dplyr, group_by)
importFrom(dplyr, summarise)
importFrom(dplyr, mutate)
importFrom(dplyr, select)
importFrom(dplyr, select_if)
importFrom(dplyr, "%>%")
importFrom(dplyr, tibble)
importFrom(dplyr, left_join)
importFrom(dplyr, mutate_if)
importFrom(dplyr, rename)
importFrom(dplyr, summarise_at)
importFrom(dplyr, ungroup)
importFrom(dplyr, rowwise)
importFrom(purrr, map)
importFrom(purrr, map_df)
importFrom(purrr, flatten)
importFrom(tidyr, spread)
importFrom(tidyr, pivot_wider)
importFrom(tidyr, pivot_longer)
importFrom(tidyr, unnest_wider)
importFrom(tidyr, unnest_longer)
importFrom(tidyr, replace_na)
importFrom(tidyr, unite)
importFrom(tidyselect, matches)
importFrom(stringr, str_interp)
importFrom(stringr, str_glue)
importFrom(stringr, str_split)
importFrom(stringr, str_c)
importFrom(stringr, str_remove_all)
importFrom(stringr, str_remove)
importFrom(stringr, str_replace)
importFrom(utils, browseURL, setTxtProgressBar, txtProgressBar)
importFrom(stats, setNames)
importFrom(pbapply, pblapply)
import(httr2)

S3method(print, fb_access_token)
S3method(print, fblimits)
S3method(fbAction, default)

</head>

<p align="center">
<a href="https://selesnow.github.io/"><img src="https://alexeyseleznev.files.wordpress.com/2017/03/as.png" height="80"></a>
</p>

# rfacebookstat <a href='https://selesnow.github.io/rfacebookstat/'><img src='https://raw.githubusercontent.com/selesnow/rfacebookstat/master/inst/logo/rfacebookstat.png' align="right" height="139" /></a>

## CRAN

[![Rdoc](http://www.rdocumentation.org/badges/version/rfacebookstat)](https://www.rdocumentation.org/packages/rfacebookstat)
[![](https://cranlogs.r-pkg.org/badges/rfacebookstat)](https://cran.r-project.org/package=rfacebookstat)
[![](https://img.shields.io/badge/august%202018-top%2040%20on%20CRAN-brightgreen.svg)](https://rviews.rstudio.com/2018/09/26/august-2018-top-40-new-packages/)

## Краткое описание.

Пакет для загрузки данных из [Marketing API Facebook](https://developers.facebook.com/docs/marketing-api) в R, а так же с помощью пакета вы можете управлять доступами пользователей к рекламный аккаунтам на Facebook.

## Достижения

1. rfacebookstat попал в [top 40 пакетов](https://rviews.rstudio.com/2018/09/26/august-2018-top-40-new-packages/), опубликованных на CRAN в августе 2018 года.

[![](https://img.shields.io/badge/august%202018-top%2040%20on%20CRAN-brightgreen.svg)](https://rviews.rstudio.com/2018/09/26/august-2018-top-40-new-packages/)

## Поддержать проект
Вы можете поддержать проект любой произвольной суммой перейдя по этой [ссылке](https://secure.wayforpay.com/button/b6dd4a7083fe0).

## Установка пакета rfacebookstat
Для установки пакета запустите приведённый ниже код в RStudio или R консоли.
Установка из главного репозитория CRAN:
```r
install.packages("rfacebookstat")
```
Устновка наиболее актульной dev версии пакета:
```r
if(!"devtools" %in% installed.packages()[,1]){install.packages("devtools")}
devtools::install_github('selesnow/rfacebookstat')
```

### Ссылки
1. [Документация по работе с пакетом rfacebookstat](https://selesnow.github.io/rfacebookstat//).
2. Баг репорты, предложения по доработке и улучшению функционала rfacebookstat оставлять [тут](https://github.com/selesnow/rfacebookstat/issues). 
3. [Список релизов](https://github.com/selesnow/rfacebookstat/releases).

### Автор пакета
Алексей Селезнёв, Head of analytics dept. at [Netpeak](https://netpeak.us/)
<Br>email: selesnow@gmail.com
<Br>skype: selesnow
<Br>facebook: [facebook.com/selesnow](https://www.facebook.com/selesnow)
<Br>blog: [alexeyseleznev.wordpress.com](https://alexeyseleznev.wordpress.com/)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# 
# fb_data <- fbGetMarketingStat(
#                   level              = "campaign",
#                   fields             = "campaign_name,
#                                         impressions,
#                                         clicks,
#                                         spend,
#                                         actions",
#                   date_start         = "2019-05-01",
#                   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# 
# fb_data_breakdowns <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   breakdowns         = "region",
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# 
# fb_data_action_breakdowns <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   action_breakdowns  = "action_reaction",
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# 
# fb_data_action_breakdowns <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   action_breakdowns  = "action_reaction",
#   attribution_window = c('default', '1d_view', '28d_view', '28d_click'),
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# fb_data <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   filtering          = "impressions LESS_THAN 5000",
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# fb_data <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   filtering          = "publisher_platform IN instagram,facebook",
#   breakdowns         = "publisher_platform",
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# 
# options(rfacebookstat.access_token = "ваш токен",
#         rfacebookstat.accounts_id  = "act_000000000")
# fb_data <- fbGetMarketingStat(
#   level              = "campaign",
#   fields             = "campaign_name,
#                         impressions,
#                         clicks,
#                         spend,
#                         actions",
#   filtering          = c("clicks LESS_THAN 500", "impressions GREATER_THAN 1000"),
#   date_start         = "2019-05-01",
#   date_stop          = "2019-05-10")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
# library(rfacebookstat)

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# library(googleAnalyticsR)
# 
# # fb options
# options(rfacebookstat.username = 'your_facebook_username')
# 
# # auth
# fbAuth()
# ga_auth(email = "google_analytics@gmail.com")
# 
# # read data from facebook
# cost_data <- fbGetCostData(accounts_id = 1111111,
#                            date_start  = '2020-03-01',
#                            date_stop   = '2020-03-10',
#                            utm_source  = 'fb',
#                            utm_medium  = 'cpc')
# 
# # upload into GA source
# ga_custom_upload_file(accountId          = 2222222,
#                       webPropertyId      = "UA-2222222-1",
#                       customDataSourceId = 'M_XXXXX_Qudk-8g',
#                       cost_data)

fbGetAdAccountsConversions <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                                       api_version  = getOption("rfacebookstat.api_version"), 
                                       username     = getOption("rfacebookstat.username"),
                                       token_path   = fbTokenPath(),
                                       access_token = getOption("rfacebookstat.access_token")) {
  
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
      access_token <- Sys.getenv("RFB_API_TOKEN")    
    } else {
      access_token <- fbAuth(username   = username, 
                             token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  # load account lists
  if ( is.null(accounts_id) ) {
    message("Loading accounts list.")
    accounts_id <- suppressMessages( fbGetUserAdAccounts()$id )
    message("...Loading conversions from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  
  #Create result data frame
  result    <- list()
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  # loading
  for ( account_id in  accounts_id ) {
    
    
    url <- str_interp("https://graph.facebook.com/${api_version}/${account_id}/customconversions")
    
    answer  <- GET(url, 
                   query = list(fields       = "d,name,rule,aggregation_rule,creation_time,custom_event_type,default_conversion_value,description,event_source_type,first_fired_time,is_archived,is_unavailable,last_fired_time,pixel,retention_days,account_id,business",
                                limit        = 150,
                                access_token = access_token))
  
    # attr
    rq_ids       <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers  <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
    
    account_custom_conversion <- content(answer, as = "parsed", encoding = "UTF-8")
    
    
    if(!is.null(account_custom_conversion$error)) {
      error <- account_custom_conversion$error
      message("\n",account_custom_conversion$error,"\n")
    }
    
    
    result <- append(result, 
                     lapply( account_custom_conversion$data, fbParserAdConversions))
    
    #Paging
    while ( !is.null(account_custom_conversion$paging$`next`) ) {
      
      link      <- account_custom_conversion$paging$`next`
      answer    <- GET(link)
      
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
      
      account_custom_conversion <- content(answer, as = "parsed", encoding = "UTF-8")
      result <- append(result, 
                       lapply( account_custom_conversion$data, fbParserAdConversions))
      
    }
    
    if ( exists("pb") ) {
      pb_step <- pb_step + 1
      utils::setTxtProgressBar(pb, pb_step)
    }

  }
  
  result <- map_df(result, flatten)
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(result)
}

# helpers 
numsum <- function(x) return(as.numeric(x) %>% sum)
ga_date_format <- function(x) format(as.Date(x), "%Y%m%d")

# fix variables 
campaign_name <- NULL
adset_name <- NULL
ad_name <- NULL
spend <- NULL
impressions <- NULL
clicks <- NULL

# main function
fbGetCostData <- 
      function(accounts_id = getOption("rfacebookstat.accounts_id"),
               date_start         = Sys.Date() - 30,
               date_stop          = Sys.Date(),
               utm_source         = "facebook",
               utm_medium         = "cpc",
               username           = getOption("rfacebookstat.username"),
               token_path         = fbTokenPath(),
               access_token       = getOption("rfacebookstat.access_token")) {
        
        
        
        data <- fbGetMarketingStat(accounts_id  = accounts_id, 
                                   level        = "ad",
                                   fields       = "campaign_name,
                                                   adset_name,
                                                   ad_name,
                                                   impressions,
                                                   clicks,
                                                   spend",
                                   date_start   = date_start,
                                   date_stop    = date_stop, 
                                   username     = username,
                                   token_path   = token_path,
                                   access_token = access_token)
        
        if ( nrow(data) == 0 ) {
          
          warning(str_interp('There is no data in your account (${accounts_id}) for the specified period (${date_start} - ${date_stop})'))
          
        } else {
          
          data <- group_by(data, 
                           date_start, campaign_name, adset_name, ad_name) %>%
                  summarise_at( c("spend", "impressions", "clicks"),
                                numsum) %>%
                  ungroup() %>%
                  mutate("ga:date"        = ga_date_format(date_start),
                         "ga:medium"      = utm_medium, 
                         "ga:source"      = utm_source) %>%
                  rename("ga:campaign"    = campaign_name,
                         "ga:keyword"     = adset_name,
                         "ga:adContent"   = ad_name,
                         "ga:adCost"      = spend,
                         "ga:impressions" = impressions,
                         "ga:adClicks"    = clicks) %>%
                  select("ga:date",
                         "ga:medium",
                         "ga:source",
                         "ga:adClicks",
                         "ga:adCost",
                         "ga:impressions",
                         "ga:campaign",
                         "ga:keyword",
                         "ga:adContent") %>%
                  mutate_if( is.character, iconv, to = "UTF-8" )
          
        }
        
        return(data)
      }

fbGetBusinessManagers <- function(api_version  = getOption("rfacebookstat.api_version"), 
                                  username     = getOption("rfacebookstat.username"),
                                  token_path   = fbTokenPath(),
                                  access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  #Check of fill out access_token
  if(is.null(access_token)){
    stop("access_token id require argument!")
  }
  	
  # attributes
  rq_ids      <- list()
  out_headers <- list()

  #Create result data frame
  result <- tibble()
 
  #Compose query string
  QueryString <- paste0("https://graph.facebook.com/",api_version,"/me/businesses?fields=id,name,primary_page,creation_time,created_by&access_token=",access_token)

  #Send query to API server
  answer <- GET(QueryString)
  
  # attr
  rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
  out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
	
  #Parse result
  raw <- content(answer, "parse", "application/json", encoding = "UTF-8")

  #Check error
  if(!is.null(raw$error)){
    stop(raw$error)
  }
  
  #Add data to result data frame
  temp_data <- map_df(raw$data, fbParserBM)
  result    <- bind_rows(result, temp_data)
  
  #Pagination
  while(!is.null(raw$paging$`next`)){
    QueryString <- raw$paging$`next`
    answer <- GET(QueryString)
	
	  # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
	
    raw <- content(answer, "parse", "application/json", encoding = "UTF-8")
    #Add data to result data frame
    temp_data <- map_df(raw$data, fbParserBM)
    result    <- bind_rows(result, temp_data)
  }

  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
return(result)
}

.onAttach <- function(lib, pkg,...){
  packageStartupMessage(rfacebookstatWelcomeMessage())
  
  packageStartupMessage("rfacebookstat presets:")
  
  ## token path
  packageStartupMessage("...Set rfacebookstat token_path: ", appendLF = F)
  if ( Sys.getenv("RFB_TOKEN_PATH") != "" ) {
    packageStartupMessage("success")
  } else {
    packageStartupMessage("none")
  }
  
  ## username
  packageStartupMessage("...Set rfacebookstat username: ", appendLF = F)
  if ( Sys.getenv("RFB_USER") != "" ) {
    packageStartupMessage("success")
  } else {
    packageStartupMessage("none")
  }
  
  ## access token
  packageStartupMessage("...Set rfacebookstat access_token: ", appendLF = F)
  if ( Sys.getenv("RFB_API_TOKEN") != "" ) {
    packageStartupMessage("success")
  } else {
    packageStartupMessage("none")
  }
  
  # api version
  packageStartupMessage("...Set Facebook Marketing API Version: ", appendLF = F)
  packageStartupMessage(getOption("rfacebookstat.api_version"))
}

#
#

rfacebookstatWelcomeMessage <- function(){
  # library(utils)
  
  paste0("\n",
         "---------------------\n",
         "Welcome to rfacebookstat version ", utils::packageDescription("rfacebookstat")$Version, "\n",
         "\n",
         "Author:           Alexey Seleznev (Head of analytics dept at Netpeak).\n",
         "Telegram channel: https://t.me/R4marketing \n",
         "YouTube channel:  https://www.youtube.com/R4marketing/?sub_confirmation=1 \n",
         "Email:            selesnow@gmail.com\n",
         "Site:             https://selesnow.github.io \n",
         "Blog:             https://alexeyseleznev.wordpress.com \n",
         "Facebook:         https://facebook.com/selesnown \n",
         "Linkedin:         https://www.linkedin.com/in/selesnow \n",
         "\n",
         "Type ?rfacebookstat for the main documentation.\n",
         "The github page is: https://github.com/selesnow/rfacebookstat/\n",
		 "Package site: https://selesnow.github.io/rfacebookstat/\n",
         "\n",
         "Suggestions and bug-reports can be submitted at: https://github.com/selesnow/rfacebookstat/issues\n",
         "Or contact: <selesnow@gmail.com>\n",
         "\n",
         "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(rfacebookstat))\n",
         "---------------------\n"
  )
}

    
.onLoad <- function(libname, pkgname) {
  
  # check environment variables
  ## token path
  if ( Sys.getenv("RFB_TOKEN_PATH") != "" ) {
    
    fb_token_path <- Sys.getenv("RFB_TOKEN_PATH")
    
  } else {
    
    fb_token_path <- NULL
    
  }
  
  ## username
  if ( Sys.getenv("RFB_USER") != "" ) {
    
    fb_user <- Sys.getenv("RFB_USER")
    
  } else {
    
    fb_user <- NULL
    
  }
  
  ## access token
  if ( Sys.getenv("RFB_API_TOKEN") != "" ) {
    
    fb_token <- Sys.getenv("RFB_API_TOKEN")
    
  } else {
    
    fb_token <- NULL
    
  }
  
  op <- options()
  op.rfacebookstat <- list(rfacebookstat.api_version  = "v25.0",
                           rfacebookstat.access_token = fb_token,
                           rfacebookstat.accounts_id  = NULL,
                           rfacebookstat.business_id  = NULL,
                           rfacebookstat.token_path   = fb_token_path,
                           rfacebookstat.username     = fb_user,
                           rfacebookstat.app_id       = 176943372709235,
                           rfacebookstat.app_secret   = "94b55b4f2396e50f6e8780377fe1bab6")
					   
  toset <- !(names(op.rfacebookstat) %in% names(op))
  if (any(toset)) options(op.rfacebookstat[toset])
  
  invisible()
}

fbGetAdCreative <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                            filtering    = NULL,
                            api_version  = getOption("rfacebookstat.api_version"),
                            username     = getOption("rfacebookstat.username"),
                            token_path   = fbTokenPath(),
                            access_token = getOption("rfacebookstat.access_token"),
                            limit        = 50){
	
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  res <- list()
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading adscreatives from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  # filters handing to JSON
  if ( ! is.null(filtering) ) {
    if ( ! validate(filtering)[[1]] ) {
      
      filters <- list()
      
      for ( f in filtering ) {
        temp_fil    <- str_split(f, " ")
        temp_filter <- list(field    = temp_fil[[1]][1],
                            operator = temp_fil[[1]][2])
        
        if ( temp_fil[[1]][2] %in% c("IN_RANGE", "NOT_IN_RANGE", "IN", "NOT_IN") ) {
          temp_filter$value <- str_split(temp_fil[[1]][3], ",")[[1]]
        } else {
          temp_filter$value <- temp_fil[[1]][3]
        }
        filters <- append(filters, list(temp_filter))
      }
      filtering <- toJSON(filters, auto_unbox = T)
    }
  }
  
  for( account_id in accounts_id ) {
      
    
      url <- str_interp("https://graph.facebook.com/${api_version}/${account_id}/adcreatives")
    
      api_answer  <- GET(url, 
                         query = list(fields       = "ad_id,name,title,body,status,adlabels,call_to_action_type,link_url,link_destination_display_url,template_url,url_tags,thumbnail_url,image_url,image_hash,object_story_spec,object_id,object_type,video_id,account_id",
                                      limit        = limit,
                                      filtering    = filtering,
                                      access_token = access_token))
      
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
      
      pars_answer <- content(api_answer, as = "parsed")
      
      if(!is.null(pars_answer$error)) {
        error <- pars_answer$error
        stop("\n",pars_answer$error,"\n")
      }
      
      # pars
      res <- append(res, 
                    lapply( pars_answer$data, fbParserAdCreatives ))
      
      # paging
      while (!is.null(pars_answer$paging$`next`)) {
        
        api_answer  <- GET(pars_answer$paging$`next`)
        pars_answer <- content(api_answer, as = "parsed")
        
        if(!is.null(pars_answer$error)) {
          error <- pars_answer$error
          stop("\n",pars_answer$error,"\n")
        }
        
        # pause between query
        Sys.sleep(0.2)  
        
        # pars
        res <- append(res, 
                      lapply( pars_answer$data, fbParserAdCreatives ))
        
      }
      
      # pause between query
      Sys.sleep(0.2)  
      
      # set progress bar
      if( pgbar ) {
        pb_step <- pb_step + 1
        setTxtProgressBar(pb, pb_step)
      }
  }
  

  res <- map_df(res, flatten)
  
  # set attributes
  attr(res, "request_ids") <- rq_ids
  attr(res, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(res)
}

fbDeleteAdAccountUsers <-  function(user_ids     = NULL, 
                                    accounts_id  = getOption("rfacebookstat.accounts_id"),
									api_version  = getOption("rfacebookstat.api_version"),
									username     = getOption("rfacebookstat.username"),
									token_path   = fbTokenPath(),
									access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  if(is.null(accounts_id)|is.null(access_token)){
    stop("Arguments accounts_id and access_token is require.")
  }
  
  for(account_id in accounts_id){
    
    for(uid in user_ids){
      print(paste0("Account ",account_id))
      QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account_id,"/users/",uid,"?access_token=",access_token)
      ans <- httr::DELETE(QueryString)
      ans <- content(ans)
      print(ans)
      Sys.sleep(3)
    }
  }
}

# fetchBy date range ------------------------------------------------------
fbDateRanges <- function(start, end, by) {
  start <- as.Date(start)
  end <- as.Date(end)
  by <- tolower(by)
  by <- match.arg(by, c("day", "week", "month", "quarter", "year"))
  dates <- seq.Date(start, end, by = by)
  res <- cbind(start = as.character(dates),
               end = as.character(c(dates[-1] - 1, end)))
  as.data.frame(res, stringsAsFactors = F)
}


# main fun - helper -------------------------------------------------------
fbGetMarketingStatHelper <-
  function(accounts_id        = getOption("rfacebookstat.accounts_id"),
           sorting            = NULL,
           level              = "account",
           breakdowns         = NULL,
           action_breakdowns  = NULL,
           fields             = "account_id,campaign_name,impressions,clicks,reach,spend",
           filtering          = NULL,
           date_start         = NULL,
           date_stop          = NULL,
           date_preset        = 'last_30d',
           attribution_window = NULL,
           use_unified_attribution_setting = FALSE,
           api_version        = getOption("rfacebookstat.api_version"),
           action_report_time = NULL,
           interval           = "day",
           use_account_attribution_setting = FALSE,
           console_type       = "progressbar",
           request_speed      = "normal",
           fetch_by           = NULL,
           username           = getOption("rfacebookstat.username"),
           token_path         = fbTokenPath(),
           access_token       = getOption("rfacebookstat.access_token")){
    
   # auth 
   if ( is.null(access_token) ) {    
    
     if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	 } else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
     }
   }
    
    if ( inherits(access_token, "fb_access_token") ) {
      
      access_token <- access_token$access_token
      
    }
    
    # Check start time
    start_time <- Sys.time()
    
    # Create result DF
    result <- tibble()
    
    # attributes
    rq_ids      <- list()
    out_headers <- list()
    
    # 
    # clear field list
    fields <- gsub("[\\s\\n\\t]", 
                   "", 
                   fields, 
                   perl = TRUE)
    
    # clear action breakdowns
    action_breakdowns <- gsub("[\\s\\n\\t]", 
                              "", 
                              action_breakdowns, 
                              perl = TRUE)
    
    if ( ! is.null(attribution_window) ) {
      
      attribution_window <- tolower(attribution_window) %>% toJSON()
      
    }
	
	# check account format
	accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
    
    # answer object class detect
    answer_class <- "actions"
    
    # filters handing to JSON
    if ( ! is.null(filtering) ) {
      if ( ! validate(filtering)[[1]] ) {
        
        filters <- list()
        
        for ( f in filtering ) {
          temp_fil    <- str_split(f, " ")
          temp_filter <- list(field    = temp_fil[[1]][1],
                              operator = temp_fil[[1]][2])
          
          if ( temp_fil[[1]][2] %in% c("IN_RANGE", "NOT_IN_RANGE", "IN", "NOT_IN") ) {
            temp_filter$value <- str_split(temp_fil[[1]][3], ",")[[1]]
          } else {
            temp_filter$value <- temp_fil[[1]][3]
          }
          filters <- append(filters, list(temp_filter))
        }
        filtering <- toJSON(filters, auto_unbox = T)
      }
    }
    
    # time_increment
    if ( interval == "overall" ) {
      #dates_from <- as.Date(date_start)
      #dates_to   <- as.Date(date_stop)
      time_increment <- 'all_days' 
      
    } else if ( interval == 'day' ) {
      
      time_increment <- 1
      
    } else if ( interval == 'weekly' ) {
      
      time_increment <- 7
    
    } else {
      
      time_increment <- interval
      
    }
    
    # time_range
    if ( !is.null(date_start) & !is.null(date_stop) ) {
      
      time_range <- str_interp("{\"since\":\"${date_start}\",\"until\":\"${date_stop}\"}")
      
    } else {
      
      time_range <- NULL
      
    }
    
    # use_account_attribution_setting
    if ( use_account_attribution_setting ) {
      
      use_account_attribution_setting <- 'true'
      
    } else {
      
      use_account_attribution_setting <- 'false'
      
    }
    
    # use_account_attribution_setting
    if ( use_unified_attribution_setting ) {
      
      use_unified_attribution_setting <- 'true'
      
    } else {
      
      use_unified_attribution_setting <- 'false'
      
    }
    
    
    #################################################
    # OLD VERSION OF CODE, WITH OUT time_range AND  #
    # time_increment                                #
    # date: 2020-09-24                              #
    #################################################
    
    # else {
    #   #Check dates interval
    #   dates_from <- seq.Date(as.Date(date_start), as.Date(date_stop), by = interval)
    #   dates_to   <- as.Date(dates_from - 1)
    #   dates_to   <- c(as.Date(dates_to[-1]),as.Date(date_stop))
    # 
    # }
    # # Create time interval data frame
    # dates_df   <- data.frame(dates_from = dates_from,
    #                          dates_to   = dates_to)
    
    #request step pause
    if(request_speed %in% c("fast","normal","slow")){
      pause_time <- switch(request_speed,
                           "fast" = 0,
                           "normal" = 1 / 5,
                           "slow" = 2)
    } else if(is.numeric(request_speed)|is.integer(request_speed)){
      pause_time <- request_speed
    } else {
      pause_time <- 0
    }
    
    #Check query number
    if(length(accounts_id) < 2){
      console_type <- "message"  
    }
    
    if(console_type == "progressbar"){      
      #Progress settings
      pb_step <- 1
      pb <- utils::txtProgressBar(pb_step, length(accounts_id), style = 3,title = "Loading:", label = "load" )}
    
    #API request counter
    request_counter <- 0
    error_counter   <- 0
    
    for(i in 1:length(accounts_id)) {
      
      #################################################
      # OLD VERSION OF CODE, WITH OUT time_range AND  #
      # time_increment                                #
      # date: 2020-09-24                              #
      #################################################
      
      # #Intervals flatten
      # for(dt in 1:nrow(dates_df)){
        
      #Create query string
      QueryString <- str_interp("https://graph.facebook.com/${api_version}/${accounts_id[i]}/insights?")
        
      #Progresbar step
      if(console_type == "progressbar"){
        pb_step <- pb_step + 1
        utils::setTxtProgressBar(pb, pb_step)}
        
      #Send API request
      answer <- httr::GET(QueryString,
                          query = list(sort                       = sorting,
                                       level                      = level,
                                       breakdowns                 = breakdowns,
                                       action_breakdowns          = action_breakdowns,
                                       fields                     = fields,
                                       filtering                  = filtering,
                                       action_report_time         = action_report_time,
                                       action_attribution_windows = attribution_window,
                                       time_increment             = time_increment,
                                       time_range                 = time_range,
                                       date_preset                = date_preset,
                                       use_account_attribution_setting = use_account_attribution_setting,
                                       use_unified_attribution_setting = use_unified_attribution_setting,
                                       limit                      = 1000,
                                       access_token               = access_token))
        
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
        
      # check limit
      queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, pause_time  )
        
      # reapet query if out of apilimit
      while ( queryrep ) {
          
          answer <- httr::GET(QueryString,
                              query = list(sort                       = sorting,
                                           level                      = level,
                                           breakdowns                 = breakdowns,
                                           action_breakdowns          = action_breakdowns,
                                           fields                     = fields,
                                           filtering                  = filtering,
                                           action_report_time         = action_report_time,
                                           action_attribution_windows = attribution_window,
                                           time_increment             = 1,
                                           time_range                 = time_range,
                                           date_preset                = date_preset,
                                           use_account_attribution_setting = use_account_attribution_setting,
                                           limit                      = 1000,
                                           access_token               = access_token))
          
          # attr
          rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
          out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
          
          queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, pause_time )
        }
        
      request_counter <- request_counter + 1
        
      #Parse result
      answerobject <- httr::content(answer, as = "parsed")
        
      #Request step pause
      Sys.sleep(pause_time)
        
      #Check answer on errors
      if (!is.null(answerobject$error)) {
        
         #Add error in error counter
         error_counter <- error_counter + 1
         if (answerobject$error$message == "(#17) User request limit reached") {
            #First attempt
            attempt <- 1
            if(console_type == "message"){
              message("WARNING: User request limit reached", appendLF = T)
              message("Apply the mechanism for circumvention of the limit", appendLF = T)
              message("Wait few minutes.", appendLF = T)}
            #Start cycle
            while(attempt <= 6){
              if(console_type == "message"){
                message(paste0("attempt number: ",attempt), appendLF = T)}
              
              #Wait one minute and repaete
              Sys.sleep(61)
              
              #Repeate API request
              answer <- httr::GET(QueryString,
                                  query = list(sort               = sorting,
                                               level              = level,
                                               breakdowns         = breakdowns,
                                               action_breakdowns  = action_breakdowns,
                                               fields             = fields,
                                               filtering          = filtering,
                                               action_report_time = action_report_time,
                                               action_attribution_windows = attribution_window,
                                               time_increment     = 1,
                                               time_range         = time_range,
                                               date_preset        = date_preset,
                                               use_account_attribution_setting = use_account_attribution_setting,
                                               limit              = 1000,
                                               access_token       = access_token))
              
            # attr
            rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
            out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
              
              request_counter <- request_counter + 1
              answerobject <- httr::content(answer, as = "parsed")
              
              #If many limits up pause time
              if(error_counter >= 3 & pause_time < 5){
                if(console_type == "message"){
                  message("WARNING: More 3 limits error, magnified pause time on 1.5", appendLF = T)}
                pause_time <- pause_time * 1.5
              }
              
              #Check new answer
              if(is.null(answerobject$error$message)) {
                if(console_type == "message"){
                  message("Problem fixed. Continue data collection", appendLF = T)}
                break}
              #Add error in error counter
              error_counter <- error_counter + 1
              #Next attempt
              attempt <- attempt + 1
            }
          }
          
          #Check other questions  
          if(!is.null(answerobject$error)){
            error <- answerobject$error
            error_msg <- answerobject$error$message
            if ( !is.null(answerobject$error$error_user_title) ) error_msg <- paste0(answerobject$error$error_user_title, ": ", error_msg)
            if ( !is.null(answerobject$error$error_user_msg) )   error_msg <- paste0(error_msg, ". ", answerobject$error$error_user_msg)
            stop(error_msg)}
          
      }
      
        # action breakdown handing
        if ( any(c("actions", "action_values", "conversions", "conversion_values") %in% unlist(str_split(fields, ","))) ) {
          
          # switch functions
          class(answerobject) <- answer_class
          
          #Adding data to result
          tempData <- fbAction(answerobject)
          
          result   <- bind_rows(result, tempData)
          
          if (exists("tempData")) {
            rm(tempData)
          }
          
        } else {
          #Adding data to result
          tempData <- bind_rows(answerobject$data)
          result   <- bind_rows(result, tempData)
        }
        
        #Pagination
        while (!is.null(answerobject$paging$`next`)) {
          QueryString <- answerobject$paging$`next`
          answer <- httr::GET(QueryString)
          
          # check limit
          queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, pause_time )
          
          # reapet query if out of apilimit
          while ( queryrep ) {
            
            answer <- httr::GET(QueryString)
            
            # attr
            rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
            out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
            
            queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, pause_time )
            
          }
          
          request_counter <- request_counter + 1
          answerobject <- httr::content(answer, as = "parsed")
          
          #Request step pause
          Sys.sleep(pause_time)
          
          #Check answer on errors
          if (!is.null(answerobject$error)) {
            #Add error in error counter
            error_counter <- error_counter + 1
            if (answerobject$error$message == "(#17) User request limit reached") {
              #First attempt
              attempt <- 1
              if(console_type == "message"){
                message("WARNING: User request limit reached", appendLF = T)
                message("Apply the mechanism for circumvention of the limit", appendLF = T)
                message("Wait few minutes.", appendLF = T)}
              #Start cycle
              while(attempt <= 5){
                if(console_type == "message"){
                  message(paste0("attempt number: ",attempt), appendLF = T)}
                
                #Wait one minute and repaete
                Sys.sleep(61)
                
                #Repeate API request
                answer <- httr::GET(QueryString)
                request_counter <- request_counter + 1
                answerobject <- httr::content(answer, as = "parsed")
                
                #Check new answer
                if(is.null(answerobject$error$message)) {
                  if(console_type == "message"){
                    message("Problem fixed. Continue data collection", appendLF = T)}
                  break}
                #Add error in error counter
                error_counter <- error_counter + 1
                #Next attempt
                attempt <- attempt + 1
              }
            }
          }
          
          # action breakdown handing
          if ( "actions" %in% unlist(str_split(fields, ",")) ) {
            
            # switch functions
            class(answerobject) <- answer_class
            
            #Adding data to result
            tempData <- fbAction(answerobject)
            
            result   <- bind_rows(result, tempData)
            
            if (exists("tempData")) {
              rm(tempData)
            }
            
          } else {
            #Adding data to result
            tempData <- bind_rows(answerobject$data)
            result   <- bind_rows(result, tempData,)
          }
        }
      }
    
    # prepare output
    # unnesting_values
    vlist <- select_if(result, is.list) %>% names()

    for ( l in vlist ) {

      result <- unnest_wider(result, l, names_sep = "_") %>%
                      select(-matches("^.*\\_action\\_type"))

    }

    # renaming
    newnames <- names(result) %>%
                str_remove_all("NA") %>%
                str_remove("^\\_") %>%
                str_replace("\\_\\_", "\\_") %>%
                str_remove("\\_value$")
   
    names(result) <- newnames
    
    # set attributes
    attr(result, "request_ids") <- rq_ids
    attr(result, "headers")     <- out_headers
    
    if(console_type == "progressbar"){  
      #Progressbar close
      utils::setTxtProgressBar(pb, length(accounts_id))
      close(pb)}
    
    #Out messages
    packageStartupMessage("-----------------------------------------------------", appendLF = T)
    packageStartupMessage("Data loaded successfully!", appendLF = T)
    packageStartupMessage(paste0("Loaded ",nrow(result)," rows."), appendLF = T)
    packageStartupMessage(paste0("Sended ",request_counter," API requests."), appendLF = T)
    if(error_counter > 0) packageStartupMessage(paste0(error_counter," errors of limit request."), appendLF = T)
    if(error_counter > 0) packageStartupMessage(paste0("Error rate ",round(error_counter / request_counter * 100,2),"%"), appendLF = T)
    packageStartupMessage(paste0("Total processing time ",round(difftime(Sys.time(), start_time, units = "secs"), 0) ," seconds."), appendLF = T)
    packageStartupMessage("-----------------------------------------------------", appendLF = T)
    return(result)
  }


# cycle_fun ---------------------------------------------------------------
fbGetMarketingStat <-
  function(accounts_id        = getOption("rfacebookstat.accounts_id"),
           sorting            = NULL,
           level              = "account",
           breakdowns         = NULL,
           action_breakdowns  = NULL,
           fields             = "account_id,campaign_name,impressions,clicks,reach,spend",
           filtering          = NULL,
           date_start         = NULL,
           date_stop          = NULL,
           date_preset        = 'last_30d',
           attribution_window = NULL,
           api_version        = getOption("rfacebookstat.api_version"),
           action_report_time = NULL,
           interval           = "day",
           use_unified_attribution_setting = FALSE,
           use_account_attribution_setting = FALSE,
           console_type       = "progressbar",
           request_speed      = "normal",
           fetch_by           = NULL,
           username           = getOption("rfacebookstat.username"),
           token_path         = fbTokenPath(),
           access_token       = getOption("rfacebookstat.access_token")) {
    
    
    # FetchBy
    if ( !is.null(fetch_by) ) {
      
      #dates df
      dates <- fbDateRanges(date_start, date_stop, fetch_by)
      n <- nrow(dates)
      message("Batch processing mode is enabled.")
      message(paste0("Fetching data by ", tolower(fetch_by), ": from ", as.Date(date_start), " to ", as.Date(date_stop), "."))
      console_type <- "message" 
      
    } else {
      #dates df
      dates <- as.data.frame(list(date_start, date_stop), col.names = c("start","end"), stringsAsFactors = F)
      n <- 1
    }
    
    
    if ( n > 1 ) {
      
      # apply fetching
      result <- pblapply(1:n,
                function(X) {
                  suppressMessages( {
                  suppressPackageStartupMessages( {
                   fbGetMarketingStatHelper( 
                       accounts_id        = accounts_id,
                       sorting            = sorting,
                       level              = level,
                       breakdowns         = breakdowns,
                       action_breakdowns  = action_breakdowns,
                       fields             = fields,
                       filtering          = filtering,
                       date_start         = dates$start[X],
                       date_stop          = dates$end[X],
                       date_preset        = date_preset,
                       attribution_window = attribution_window,
                       api_version        = api_version,
                       action_report_time = action_report_time,
                       interval           = interval,
                       use_unified_attribution_setting = use_unified_attribution_setting,
                       use_account_attribution_setting = use_account_attribution_setting,
                       console_type       = console_type,
                       request_speed      = request_speed,
                       fetch_by           = fetch_by,
                       username           = username,
                       token_path         = token_path,
                       access_token       = access_token
                       )
                }
                )
          }
          )
      }
      )
      
    } else {
      
      # simple request
      result <- fbGetMarketingStatHelper( 
                      accounts_id        = accounts_id,
                      sorting            = sorting,
                      level              = level,
                      breakdowns         = breakdowns,
                      action_breakdowns  = action_breakdowns,
                      fields             = fields,
                      filtering          = filtering,
                      date_start         = dates$start,
                      date_stop          = dates$end,
                      date_preset        = date_preset,
                      attribution_window = attribution_window,
                      api_version        = api_version,
                      action_report_time = action_report_time,
                      interval           = interval,
                      use_unified_attribution_setting = use_unified_attribution_setting,
                      use_account_attribution_setting = use_account_attribution_setting,
                      console_type       = console_type,
                      request_speed      = request_speed,
                      fetch_by           = fetch_by,
                      username           = username,
                      token_path         = token_path,
                      access_token       = access_token
                    )
                      
    }
    
    # collect all data
    result <- bind_rows(result)

    # return 
    return( result )
  }

fbUpdateAdAccountUsers <- function(user_ids = NULL, 
                                   role = "advertiser", 
                                   accounts_id = getOption("rfacebookstat.accounts_id"),
                                   api_version = getOption("rfacebookstat.api_version"),
                                   username     = getOption("rfacebookstat.username"),
                                   token_path   = fbTokenPath(),
                                   access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  #Check account_id, token and uid
  if(is.null(accounts_id)|is.null(access_token)|is.null(user_ids)){
    stop("Arguments user_ids, accounts_id and access_token is require.")
  }
  
  #Check role
  role <- switch(tolower(role), 
                 "administator" = 1001,
                 "advertiser"   = 1002,
                 "ad manager"   = 1002,
                 "analyst"      = 1003,
                 "sales"        = 1004,
                 "direct sales" = 1004,
                 tolower(role))
  
  for(account_id in accounts_id){
    
    for(uid in user_ids){
      print(paste0("Add user ", uid, " to account ", account_id))
      #Compose query
      QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account_id,"/users?uid=",uid,"&role=",role,"&access_token=",access_token)
      #Send query
      ans <- httr::POST(QueryString)
      #Parse query
      ans <- content(ans)
      #Message
      print(ans)} 
      #sleep 3 second before next request
      Sys.sleep(3)
  }
}

fbGetBusinessManagersUsers <- function(business_ids  = getOption("rfacebookstat.business_id"),
                                       user_types    = c('business_users', 'system_users', 'pending_users'),
                                       api_version   = getOption("rfacebookstat.api_version"), 
                                       username      = getOption("rfacebookstat.username"),
                                       token_path    = fbTokenPath(),
                                       access_token  = getOption("rfacebookstat.access_token")) {
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
      access_token <- Sys.getenv("RFB_API_TOKEN")    
    } else {
      access_token <- fbAuth(username   = username, 
                             token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  if ( is.null(access_token) || is.null(business_ids) ) {
    stop("Arguments access_token and business_id is require.")
  }
  
  # result obj
  result <- list()
  
  # activate progress bar
  if ( length(business_ids) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(business_ids), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  # bussiness ids
  for (business_id in  business_ids) {

    # user types
    for (type in user_types) {

      url <- str_interp("https://graph.facebook.com/${api_version}/${business_id}/${type}")
      
      api_answer  <- GET(url, 
                         query = list(fields       = "id,name,business,first_name,last_name,title,role,email",
                                      limit        = 1000,
                                      access_token = access_token))
      
      pars_answer <- content(api_answer, as = "parsed")
      
      if(!is.null(pars_answer$error)) {
        error <- pars_answer$error
        stop(pars_answer$error)
      }
      
      if (length(pars_answer$data) == 0) {
        
        # set progress bar
        if( pgbar ) {
          pb_step <- pb_step + 1
          setTxtProgressBar(pb, pb_step)
        }
        
        next
        
      }
      
      
      # pars
      result <- append(result, 
                       lapply( pars_answer$data, fbParserBMUsers, user_type = type))
      
      # paging
      while (!is.null(pars_answer$paging$`next`)) {
        
        api_answer  <- GET(pars_answer$paging$`next`)
        pars_answer <- content(api_answer, as = "parsed")
        
        # pars
        result <- append(result, 
                         lapply( pars_answer$data, fbParserBMUsers, user_type = type))
        
      }
      
      # pause between query
      if (pgbar) Sys.sleep(0.2)  
      
    }
    
    # pause between query
    if (pgbar) Sys.sleep(0.2)  
    
    # set progress bar
    if( pgbar ) {
      pb_step <- pb_step + 1
      setTxtProgressBar(pb, pb_step)
    }
    
  }
  
  # to df
  result <- map_df(result, flatten)
  
  # result
  return(result)                                       
                                        
}

fbGetBusinessUserAdAccounts <- function(business_users_id = NULL, 
                                        business_id       = getOption("rfacebookstat.business_id"), 
                                        api_version       = getOption("rfacebookstat.api_version"), 
                                        username          = getOption("rfacebookstat.username"),
                                        token_path        = fbTokenPath(),
                                        access_token      = getOption("rfacebookstat.access_token")) 
{
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
      access_token <- Sys.getenv("RFB_API_TOKEN")    
    } else {
      access_token <- fbAuth(username   = username, 
                             token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  if ( is.null(access_token) || is.null(business_id) ) {
    stop("Arguments access_token and business_id is require.")
  }  
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  # check user id
  if ( is.null(business_users_id) ) {
    
    QueryString <- paste0("https://graph.facebook.com/", 
                          api_version, "/", business_id, "/business_users?fields=id,email,business,first_name,last_name,name,role,title", "&limit=150&access_token=", access_token)
    
    answer <- httr::GET(QueryString)
    
    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
    
    raw <- fromJSON(httr::content(answer, "text", "application/json", 
                                  encoding = "UTF-8"))
    
    message("user = ", raw$data$name, "\nid = ", raw$data$id)
    business_users_id <- raw$data$id
  }
  
  # load accounts
  QueryString <- paste0("https://graph.facebook.com/", 
                        api_version, "/", business_users_id, "/assigned_ad_accounts?fields=tasks,permitted_tasks", 
                        "&limit=5000&access_token=", access_token)
  
  # get answer 
  answer <- httr::GET(QueryString)
  
  # attr
  rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
  out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
  
  # pars answer
  raw <- fromJSON(httr::content(answer, "text", "application/json", 
                                encoding = "UTF-8"))
  
  # check for error
  if (length(raw$error) > 0) {
    packageStartupMessage(paste0(" - ", raw$error$code, 
                                 " - ", raw$error$message), appendLF = T)
  }
  
  # create data frame
  result <- data.frame(id = raw$data$id,
                       task = sapply(raw$data$tasks,  str_c, collapse = ","),
                       permitted_tasks = sapply(raw$data$permitted_tasks,  str_c, collapse = ","))
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  return(result)
}

# action hander 
# https://developers.facebook.com/docs/marketing-api/reference/ads-action-stats/

name        <- NULL
val         <- NULL
.           <- NULL
type        <- NULL
action_type <- NULL

# create methode
fbAction <- function(obj, ...) {
  
  UseMethod("fbAction", obj)
  
}

# cteate parser of all methods
# ============
# actions
# ============
# ============
fbAction.default <- function( obj, ... ) {
  
  actions <-
    map_df(obj$data, 
           function(.x) {
             
             nm <- names(.x)
             nm <- nm[ ! nm %in% names(.x[unlist(map(.x, is.list))]) ]
             
             other_col <- .x[nm] %>% bind_rows()
             
             if ( length(.x$actions ) > 0 ) {
               
               df_actions <-
                 .x$actions %>%
                 bind_rows() %>%
                 pivot_longer(cols = -matches("action\\_.*" ), names_to = "action_sufix", values_to = "val") %>%
                 unite(action_type, matches("action\\_.*" ), remove = T) %>%
                 replace_na(list(val = "0")) %>%
                 pivot_wider(names_from = "action_type", values_from = "val", values_fill = list("val" = "0")) %>%
                 bind_cols(other_col, .)
               
               action_df <- df_actions
             } 
             
             if ( length(.x$conversions ) > 0 ) {
               
               df_conversion <-
                 .x$conversions %>%
                 bind_rows() %>%
                 pivot_longer(cols = -matches("action\\_.*" ), names_to = "action_sufix", values_to = "val") %>%
                 unite(action_type, matches("action\\_.*" ), remove = T) %>%
                 replace_na(list(val = "0")) %>%
                 pivot_wider(names_from = "action_type", values_from = "val", values_fill = list("val" = "0"))
               
               if ( exists("action_df") ) {
                 
                 action_df <- bind_cols(action_df, df_conversion) 
                 
               } else {
                 
                 action_df <- bind_cols(other_col, df_conversion) 
                 
               }
             } 
             
             if ( length(.x$action_values ) > 0 ) {
               
               df_action_values <-
                 .x$action_values %>%
                 bind_rows() %>%
                 pivot_longer(cols = -matches("action\\_.*" ), names_to = "action_sufix", values_to = "val") %>%
                 mutate(action_type = paste0("action_values.", action_type)) %>%
                 unite(action_type, matches("action\\_.*" ), remove = T) %>%
                 replace_na(list(val = "0")) %>%
                 pivot_wider(names_from = "action_type", values_from = "val", values_fill = list("val" = "0")) 
               
               
               if ( exists("action_df") ) {
                 
                 action_df <- bind_cols(action_df, df_action_values) 
                 
               } else {
                 
                 action_df <- bind_cols(other_col, df_action_values) 
                 
               }
               
             }
             
             if ( length(.x$video_thruplay_watched_actions ) > 0 ) {
               
             df_video <-
               .x[['video_thruplay_watched_actions']] %>%
               bind_rows() %>%
               pivot_longer(cols = -matches("action\\_.*" ), names_to = "action_sufix", values_to = "val") %>%
               mutate(action_type = paste0("video_thruplay", action_type)) %>%
               unite(action_type, matches("action\\_.*" ), remove = T) %>%
               replace_na(list(val = "0")) %>%
               pivot_wider(names_from = "action_type", values_from = "val", values_fill = list("val" = "0")) 
             
             if ( exists("action_df") ) {
               
               action_df <- bind_cols(action_df, df_video) 
               
             } else {
               
               action_df <- bind_cols(other_col, df_video) 
               
             }
             }
             
             if ( length( .x$actions ) + length( .x$action_values ) + length(.x$video_thruplay_watched_actions ) + length(.x$conversions ) == 0 ) {
               
               other_col
               
             } else {
               
               action_df
               
             }
             
           }
    )
  
  return(actions)
}

fbGetAdAccounts <- function(source_id    = getOption("rfacebookstat.business_id"), 
                            api_version  = getOption("rfacebookstat.api_version"), 
                            username     = getOption("rfacebookstat.username"),
                            token_path   = fbTokenPath(),
                            access_token = getOption("rfacebookstat.access_token")) {
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
      access_token <- Sys.getenv("RFB_API_TOKEN")    
    } else {
      access_token <- fbAuth(username   = username, 
                             token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  
  # Check source
  if(is.null(source_id)){
    source_id <- "me"
  }
  
  # Define endpoint, joint operation with v2.11 and v2.10
  if(source_id == "me"){
     end_points <- "adaccounts"
  } else {
     end_points <- c("owned_ad_accounts", "client_ad_accounts")
  }
  
  # Create result data frame
  result <- tibble()
  
  # start endpoint_cycle
  for ( end_point in end_points ) {
    
    # compose
    url <- str_interp("https://graph.facebook.com/${api_version}/${source_id}/${end_point}")
    
    # send request
    answer  <- GET(url, 
                   query = list(fields       = "id,name,account_id,account_status,amount_spent,created_time,balance,business_name,media_agency,currency,owner,partner,age,timezone_name,disable_reason,spend_cap",
                                limit        = 100,
                                access_token = access_token))

    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
    
    # Parse result
    raw <- content(answer, "parse", "application/json", encoding = "UTF-8")
    
    # Check error
    if(!is.null(raw$error)) {
      stop(raw$error$message)
    }
    
    #Add data to result data frame
    temp_data <- map_df(raw$data, flatten)
    result    <- bind_rows(result, temp_data)
    # Paging
    while(!is.null(raw$paging$`next`)) {
      
      QueryString <- raw$paging$`next`
      answer <- GET(QueryString)
      
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
      
      raw <- content(answer, "parse", "application/json", encoding = "UTF-8")
      #Add data to result data frame
      temp_data <- map_df(raw$data, flatten)
      result    <- bind_rows(result, temp_data)

   }
  }
  
  # Transform to numeric
  result$balance      <- as.numeric(result$balance)
  result$balance      <- result$balance / 100
  result$amount_spent <- as.numeric(result$amount_spent)
  result$amount_spent <- result$amount_spent / 100
  result$spend_cap    <- as.numeric(result$spend_cap)
  result$spend_cap    <- result$spend_cap / 100
  
  # joining disable reason names
  result <- left_join(result,
                      fb_data.disable_reason,
                      by = c("disable_reason" = "reason_id"))
  
  result <- left_join(result,
                      fb_data.account_statuses,
                      by = c("account_status" = "status_id"))
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  return(result)
}
  

fbGetApps <-
function(accounts_id  = getOption("rfacebookstat.accounts_id"), 
         api_version  = getOption("rfacebookstat.api_version"), 
         username     = getOption("rfacebookstat.username"),
         token_path   = fbTokenPath(),
         access_token = getOption("rfacebookstat.access_token")) {
	
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading apps from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  result <- tibble()
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  for(account_id in accounts_id){
  
    QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account_id,"/applications?fields=id,name,app_name,link,logo_url,created_time,description,app_type,category,subcategory,company&access_token=",access_token)
	
    answer <- GET(QueryString)
	
	  # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
  
    answerobject <- content(answer, as = "parsed") # legacy fromJSON(answer)
	
    tempData <- map_df(answerobject$data, flatten)
  
    # add info about account
    if ( nrow(tempData) > 0 && length(accounts_id) > 1 ) {
      
      tempData$account_id <- account_id
        
    }
    
    result   <- bind_rows(result, tempData)
	
    if(!is.null(answerobject$error)) {
      error <- answerobject$error
      message(message(answerobject$error))
    }
	
    if(exists("tempData")){rm(tempData)}
    
    # set progress bar
    if( pgbar ) {
      pb_step <- pb_step + 1
      setTxtProgressBar(pb, pb_step)
    }
    
  }
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(result)
}

# block for parsing helpers function
# check for null
fbNullReplacer <- function(x) {
  
  return(
    ifelse( is.null(x), 
            NA,
            x) 
  )
}

# pasing ads
fbParserAds <- function(x) {
  
  return(
    list(id                = x$id,
         name              = x$name,
         creative_id       = x$creative$id,
         adset_id          = x$adset_id,
         campaign_id       = x$campaign_id,
         account_id        = x$account_id,
         bid_amount        = fbNullReplacer(x$bid_amount) ,
         bid_type          = x$bid_type,
         configured_status = x$configured_status,
         effective_status  = x$effective_status)
  )
  
}

# pasing adsets
fbParserAdsets <- function(x) {
  
  return(
    list(id                = x$id,
         name              = x$name,
         account_id        = x$account_id,
         bid_amount        = fbNullReplacer(x$bid_amount),
         bid_strategy      = fbNullReplacer(x$bid_strategy),
         billing_event     = fbNullReplacer(x$billing_event),
         budget_remaining  = fbNullReplacer(x$budget_remaining) ,
         campaign_id       = x$campaign_id ,
         configured_status = x$configured_status,
         effective_status  = x$effective_status,
         status            = x$status,
         optimization_goal = fbNullReplacer(x$optimization_goal),
         pacing_type       = paste0(x$pacing_type, collapse = ", "),
         destination_type  = fbNullReplacer(x$destination_type),
         daily_budget      = fbNullReplacer(x$daily_budget),
         created_time      = x$created_time,
         source_adset_id   = fbNullReplacer(x$source_adset_id),
         is_adset_budget_sharing_enabled = fbNullReplacer(x$is_adset_budget_sharing_enabled)
         )
  )
  
}

# pasing pages
fbParserPages <- function(x) {
  
  return(
    list(id                = x$id,
         name              = fbNullReplacer(x$name),
         username          = fbNullReplacer(x$username),
         link              = x$link,
         general_info      = fbNullReplacer(x$general_info),
         is_owned          = x$is_owned,
         is_published      = x$is_published,
         about             = fbNullReplacer(x$about),
         business_id       = fbNullReplacer(x$business$id),
         business_name     = fbNullReplacer(x$business$name),
         category          = fbNullReplacer(x$category)
         )
    )
  
}

# pasing ad creatives
fbParserAdCreatives <- function(x) {
  
  if ( ! is.null(x$object_type) ) {
    if ( x$object_type == "VIDEO" ) {
      data_type_name <- "video_data"
    } else {
      data_type_name <- "link_data"
    }
  } else {
      data_type_name <- "unknown"
  }
  
  return(
    list(
      id               = x$id,
      name             = x$name,
      title            = fbNullReplacer(x$title),
      body             = fbNullReplacer(x$body),
      status           = x$status,
      object_type      = fbNullReplacer(x$object_type),
      url_tags         = fbNullReplacer(x$url_tags),
      account_id       = fbNullReplacer(x$account_id),
      page_id          = fbNullReplacer(x$object_story_spec$page_id), 
      link             = ifelse( ! is.null(x$object_story_spec$video_data) , 
                                 fbNullReplacer(x$object_story_spec$video_data$call_to_action$value$link), 
                                 fbNullReplacer(x$object_story_spec$link_data$link)), 
      link_nested      = ifelse(is.null(x$object_story_spec$link_data$child_attachments), 
                                NA, 
                                paste0(lapply(x$object_story_spec$link_data$child_attachments, 
                                              function (x) paste0(x$link)), collapse = ";")), 
      call_to_action   = ifelse(is.null(x$object_story_spec$link_data$call_to_action), 
                                NA, 
                                paste0(lapply(x$object_story_spec$link_data$call_to_action, 
                                              function (x) paste0(x)), collapse = ",")),
      message          = fbNullReplacer(x$object_story_spec[[data_type_name]]$message), 
      caption          = fbNullReplacer(x$object_story_spec$link_data$caption), 
      video_id         = fbNullReplacer(x$object_story_spec$video_data$video_id), 
      attachment_style = fbNullReplacer(x$object_story_spec$link_data$attachment_style), 
      description      = fbNullReplacer(x$object_story_spec$link_data$description), 
      image_hash       = fbNullReplacer(x$object_story_spec[[data_type_name]]$image_hash),
      image_url        = fbNullReplacer(x$image_url)
    )
  )
  
}

# pasing bussines managers
fbParserBM <- function(x) {
  
  return(
    list(id                       = x$id,
         name                     = x$name,
         primary_page_id          = x$primary_page$id,
         primary_page_name        = x$primary_page$name,
         created_by_id            = fbNullReplacer(x$created_by$id),
         created_by_name          = fbNullReplacer(x$created_by$name),
         created_by_business_id   = fbNullReplacer(x$created_by$business$id),
         created_by_business_name = fbNullReplacer(x$created_by$business$name)
  )
  )
  
}

# advideo parser
fbParserAdVideos <- function(x, account_id) {
  
  return(
    list(id                       = x$id,
         video_status             = fbNullReplacer(x$status$video_status),
         processing_progress      = fbNullReplacer(x$status$processing_progress),
         published                = x$published,
         source                   = x$source,
         length                   = x$length,
         icon                     = x$icon,
         content_category         = x$content_category,
         created_time             = x$created_time,
         updated_time             = x$updated_time,
         embeddable               = x$embeddable,
         embed_html               = x$embed_html,
         is_crosspost_video       = x$is_crosspost_video,
         is_crossposting_eligible = x$is_crossposting_eligible,
         is_episode               = x$is_episode,
         is_instagram_eligible    = x$is_instagram_eligible,
         account_id               = account_id)
  )
  
}

# conversions parser
fbParserAdConversions <- function(x) {
  
  return(
    list(id                       = x$id,
         name                     = x$name,
         rule                     = x$rule,
         creation_time            = x$creation_time,
         custom_event_type        = x$custom_event_type,
         default_conversion_value = x$default_conversion_value,
         event_source_type        = x$event_source_type,
         first_fired_time         = fbNullReplacer(x$first_fired_time),
         is_archived              = x$is_archived,
         is_unavailable           = x$is_unavailable,
         pixel_ids                = ifelse(is.null(x$pixel), 
                                           NA, 
                                           paste0(lapply(x$pixel, 
                                                         function (x) paste0(x)), collapse = ",")), 
         retention_days           = x$retention_days,
         account_id               = x$account_id)
  )
  
}

# business users parser
# conversions parser
fbParserBMUsers <- function(x, user_type) {
  
  return(
    list(id            = x$id,
         name          = x$name,
         business_id   = x$business$id,
         business_name = x$business$name,
         role          = x$role,
         first_name    = x$first_name,
         last_name     = x$last_name,
         email         = x$email,
         user_type     = user_type)
  )
  
}

fbGetAdAccountUsersPermissions <- function(accounts_id = getOption("rfacebookstat.accounts_id") ,
                                           api_version = getOption("rfacebookstat.api_version"),
                                           console_type = "progressbar",
                                           username     = getOption("rfacebookstat.username"),
                                           token_path   = fbTokenPath(),
                                           access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }

  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading users from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }

  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  #Create
  result <- data.frame(stringsAsFactors = F)

  accounts_id <- ifelse(grepl("^act_",accounts_id), accounts_id, paste0("act_",accounts_id))

  #Progress settings
  pb_step <- 1

  if(console_type == "progressbar" & length(accounts_id) > 1){
    pb <- utils::txtProgressBar(pb_step, length(accounts_id), style = 3)}
  else{
    console_type <- "message"
  }

  for(account in accounts_id){

    QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account,"/users","?access_token=",access_token)
    ans <- httr::GET(QueryString)
	
    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(ans)), ans$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(ans)), ans$headers$`x-fb-trace-id`))
	
    ans_pars <- httr::content(ans)

    if(console_type == "progressbar"){
      pb_step <- pb_step + 1
      utils::setTxtProgressBar(pb, pb_step)}

    if(length(ans_pars$data) == 0) next

    for(row in 1:length(ans_pars$data)){
      result <- rbind(result,
                      data.frame(name = ans_pars$data[[row]]$name,
                                 id   = ans_pars$data[[row]]$id,
                                 role = paste0(ans_pars$data[[row]]$tasks, collapse = ", "),
                                 account_id = account))
    }
  }
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  return(result)
}

fbGetUserAdAccounts <- function(user_id      = "me",
                                api_version  = getOption("rfacebookstat.api_version"), 
                                username     = getOption("rfacebookstat.username"),
                                token_path   = fbTokenPath(),
                                access_token = getOption("rfacebookstat.access_token")) {
  
                                   
  # auth 
  if ( is.null(access_token) ) {    
                                       
  if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
    access_token <- Sys.getenv("RFB_API_TOKEN")    
  } else {
    access_token <- fbAuth(username   = username, 
                           token_path = token_path)$access_token
    }
  }
                                     
  if ( inherits(access_token, "fb_access_token") ) {
  
    access_token <- access_token$access_token
                                       
  }
                                   
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  #Create result data frame
  result    <- tibble()
  
  link      <- paste0("https://graph.facebook.com/", api_version,"/", user_id, "/adaccounts", "?fields=id,name,account_id,account_status,amount_spent,balance,business_name,currency,owner&limit=5000&access_token=", access_token)
  answer    <- GET(link)
  
  # attr
  rq_ids       <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
  out_headers  <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
  
  user_account <- content(answer)
  
  if ( !is.null(user_account$error) ) {
    stop(user_account$error$message)
  }
  
  result       <- bind_rows(user_account$data)
  
  #Paging
  while ( !is.null(user_account$paging$`next`) ) {
    
    link      <- user_account$paging$`next`
    answer    <- GET(link)
    
    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
    
    user_account <- content(answer)
	
	if ( !is.null(user_account$error) ) {
       stop(user_account$error$message)
    }
	
    result    <- bind_rows(user_account$data)
    
    }

  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  return(result)
}

fbGetAdAccountUsers <- function(accounts_id  = getOption("rfacebookstat.accounts_id") ,
                                business_id  = getOption("rfacebookstat.business_id"),
                                api_version  = getOption("rfacebookstat.api_version"),
                                console_type = "progressbar",
                                username     = getOption("rfacebookstat.username"),
                                token_path   = fbTokenPath(),
                                access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading users from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  if (length(accounts_id) == 1) {
    console_type <- "message"
  }
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  #Progress settings
  pb_step <- 1
  
  if ( console_type == "progressbar" ) {
    pb <- utils::txtProgressBar(pb_step, length(accounts_id), style = 3)
  }
  
  #Check account ids
  accounts_id <- ifelse(grepl("^act_",accounts_id),accounts_id,paste0("act_",accounts_id))
  
  #result df
  result <- data.frame( role                  = character(0),
                        status                = character(0),
                        user.name             = character(0),
                        user.id               = character(0),
                        business_persona.name = character(0),
                        business_persona.id   = character(0),
                        account_id            = character(0),
                        email                 = character(0),
                        business.id           = character(0),
                        business.name         = character(0),
                        stringsAsFactors      = F)
  
  #Create counter variables
  account_number <- 0
  error_number   <- 0
  
  #Start message
  if(console_type == "message"){
    packageStartupMessage("Processing...", appendLF = T)}
  
  
  
  #start cycle
  for(account in accounts_id){
    #Progressbar step
    if(console_type == "progressbar"){
      pb_step <- pb_step + 1
      setTxtProgressBar(pb, pb_step)}
    
    if(is.na(account)|is.null(account)) next
    account_number <- account_number + 1
    
    #Compose URL hhtp request
    if(console_type == "message"){
      packageStartupMessage(account, appendLF = F)}
    
    QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account,"/assigned_users?fields=user,business_persona,tasks,email,status&business=",business_id,"&limit=150&access_token=",access_token)
    
    #Send request
    answer <- httr::GET(QueryString)
    
    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
    
    raw <- fromJSON(httr::content(answer, "text", "application/json",encoding = "UTF-8"))
    
    #Check answer on error
    if(length(raw$error) > 0){
      if(console_type == "message"){
        packageStartupMessage(paste0(" - ",raw$error$code, " - ", raw$error$message), appendLF = T)}
      error_number   <- error_number + 1
      next
    }
    
    #Parse answer and transform him to data frame
    flatten_data <- fromJSON(httr::content(answer, "text", "application/json",encoding = "UTF-8"), flatten = T)$data
    
    if(is.null(flatten_data)|length(flatten_data) == 0){
      if(console_type == "message"){
        packageStartupMessage(paste0(" - Empty userlist!"), appendLF = T)}
      error_number   <- error_number + 1
      next
    }
    
    #add account id ino data frame
    flatten_data$account_id <- account
    
    #Add all field list
    for (col in names(result)){
      if (is.null(flatten_data[[col]])) {
        flatten_data[[col]] <- NA
      }
    }
    
    #Add to result data.frame
    result <- rbind(result, flatten_data)
    
    if(console_type == "message"){
      packageStartupMessage(paste0(" - Done, ",nrow(flatten_data)," users"), appendLF = T)}
  }
  
  #Finish progressbar
  if(console_type == "progressbar"){
    pb_step <- pb_step + 1
    utils::setTxtProgressBar(pb, length(accounts_id))
    close(pb)}
	
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  #Loaded data info message
  packageStartupMessage("Done", appendLF = T)
  packageStartupMessage(paste0("Load userlist from ", account_number - error_number, " accounts."), appendLF = T)
  if(error_number > 0) packageStartupMessage(paste0("Error in ", error_number, " accounts."), appendLF = T)
  return(result)
}

fbSetUsername   <- function(username)     {options(rfacebookstat.username    = username)     ;cat(str_interp("Set username: ${username}"))}
fbSetAccount    <- function(accounts_ids) {options(rfacebookstat.accounts_id = accounts_ids) ;cat(str_interp("Set accounts_id: ${paste0(accounts_ids, collapse=', ')}"))}
fbSetBusinessId <- function(business_ids) {options(rfacebookstat.business_id = business_ids) ;cat(str_interp("Set accounts_id: ${paste0(business_ids, collapse=', ')}"))}
fbSetTokenPath  <- function(token_path)   {options(rfacebookstat.token_path  = token_path)   ;cat(str_interp("Set token_path: ${token_path}"))}
fbSetApiVersion <- function(api_version)  {options(rfacebookstat.api_version = api_version)  ;cat(str_interp("Set api_version: ${api_version}"))}

fbGetAds <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                     fields       = c("id", 
                                      "name" , 
                                      "object_url",
                                      "adlabels",
                                      "adset_id",
                                      "bid_amount",
                                      "bid_type",
                                      "campaign_id",
                                      "account_id",
                                      "configured_status",
                                      "effective_status",
                                      "creative"),
                     api_version  = getOption("rfacebookstat.api_version"),
                     username     = getOption("rfacebookstat.username"),
                     token_path   = fbTokenPath(),
                     access_token = getOption("rfacebookstat.access_token"),
                     limit        = 5000
                     ) {
	
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading ads from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  result <- list()
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  for ( account_id in accounts_id ) {
      
      api_answer <- request('https://graph.facebook.com') %>% 
        req_url_path_append(api_version) %>%
        req_url_path_append(account_id) %>% 
        req_url_path_append('ads') %>%  
        req_url_query(
          fields       = fields,
          limit        = limit,
          #filtering    = "[{'field':'ad.delivery_info','operator':'NOT_IN','value':['stupid_filter']}]",
          access_token = access_token,
          .multi = "comma"
        ) %>% 
        req_error(body = fb_error_body) %>% 
        req_retry(
          is_transient = \(resp) resp_status(resp) %in% c(429, 500, 503), 
          max_tries = 25,
          backoff = ~3
        ) %>% 
        req_perform()
      
      pars_answer <- resp_body_json(api_answer)

      # attr
      rq_ids      <- append(rq_ids, setNames(list(resp_status(last_response())), api_answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(resp_headers(last_response())), api_answer$headers$`x-fb-trace-id`))
      
      # pars_answer <- content(api_answer, as = "parsed")
      
      if(!is.null(pars_answer$error)) {
        error <- pars_answer$error$message
        stop(pars_answer$error$message)
      }
      
      if (length(pars_answer$data) == 0) {
        
        # set progress bar
        if( pgbar ) {
          pb_step <- pb_step + 1
          setTxtProgressBar(pb, pb_step)
        }
        
        next
        
      }
      
      # pars
      result <- append(result, 
                       lapply( pars_answer$data, fbParserAds ))
    
      # paging
      while (!is.null(pars_answer$paging$`next`)) {
        
        api_answer <- request(pars_answer$paging$`next`) %>% 
          req_error(body = fb_error_body) %>% 
          req_retry(
            is_transient = \(resp) resp_status(resp) %in% c(429, 500, 503), 
            max_tries = 25, 
            backoff = ~3
          ) %>% 
          req_perform()
        
        pars_answer <- resp_body_json(api_answer)
        
        # pars
        result <- append(result, 
                         lapply( pars_answer$data, fbParserAds ))
        
    }
    
    # pause between query
    if (pgbar) Sys.sleep(0.2)  
    
    # set progress bar
    if( pgbar ) {
        pb_step <- pb_step + 1
        setTxtProgressBar(pb, pb_step)
    }

}

  # to df
  result <- map_df(result, flatten)
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  
  # close pb
  if( pgbar ) close(pb)
  
  return(result)
}

fbGetLongTimeToken <-
  function(client_id = NULL,client_secret = NULL,fb_exchange_token = NULL){
    if(is.null(client_id)|is.null(client_secret)|is.null(fb_exchange_token)){
      stop("Enter your Client ID, client_secret and short time token!")
    }
    link <- paste0("https://graph.facebook.com/oauth/access_token?grant_type=fb_exchange_token&client_id=",client_id,"&client_secret=",client_secret,"&fb_exchange_token=",fb_exchange_token)
    raw_token <- GET(link)
    token_list <- content(raw_token)
    
    if(!is.null(token_list$error)) {
      error <- token_list$error
      message(error)
    }
    message("Token changed successfully")
    message("New longtime token: ", token_list$access_token)
    message("Expires in ", Sys.time() + as.numeric(token_list$expires_in, units = "secs"))
    return(token_list$access_token)
}

fbGetAdSets <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                        api_version  = getOption("rfacebookstat.api_version"),
                        username     = getOption("rfacebookstat.username"),
                        token_path   = fbTokenPath(),
                        access_token = getOption("rfacebookstat.access_token")){
  
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  result <- list()
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading adsets from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  for( account_id in accounts_id ) {
  
  QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account_id,"/adsets?fields=id,name,account_id,bid_amount,bid_strategy,billing_event,budget_remaining,campaign_id,configured_status,effective_status,status,optimization_goal,pacing_type,destination_type,daily_budget,created_time,source_adset_id,is_adset_budget_sharing_enabled&limit=500",
                                                "&filtering=[{'field':'adset.delivery_info','operator':'NOT_IN','value':['stupid_filter']}]",
                                                "&access_token=",access_token)
  
  api_answer  <- GET(QueryString)
    
  # attr
  rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
  out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
  
  pars_answer <- content(api_answer, as = "parsed")
  
  if(!is.null(pars_answer$error)) {
    error <- pars_answer$error
    stop(pars_answer$error)
  }

  tempData     <- map_df(pars_answer$data, fbParserAdsets)
  result       <- bind_rows(result, tempData)

  # paging
  while (!is.null(pars_answer$paging$`next`)) {

    api_answer  <- GET(pars_answer$paging$`next`)
    pars_answer <- content(api_answer, as = "parsed")
    
    tempData     <- map_df(pars_answer$data, fbParserAdsets)
    result       <- bind_rows(result, tempData)
  }
   
  # pause between query
  if (pgbar) Sys.sleep(0.2)  
  
  # set progress bar
  if( pgbar ) {
      pb_step <- pb_step + 1
      setTxtProgressBar(pb, pb_step)
   }
  
  }

  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(result)
}

# authorization

#' Get API facebook token.
#' @description Get API facebook token for access to facebook ads API.
#'
#' @param app_id ID of your Facebook App
#' @param scopes Permissions provide a way for your app to access data from Facebook. For detail see \href{https://developers.facebook.com/docs/permissions}{docmentation}
#'
#' @return API token
#' @export
#'
#' @examples
#' \dontrun{
#' tkn <- fbGetToken()
#' }
fbGetToken <-
  function(
    app_id = NULL,
    scopes = c("ads_read", "business_management", "pages_manage_ads", "ads_management", "public_profile") 
    ) 
  {
    
    if(is.null(app_id)) stop("Enter your app id.")
    scopes <- paste(scopes, collapse = ",")
    utils::browseURL(paste0("https://www.facebook.com/dialog/oauth?client_id=",app_id  ,"&display=popup&redirect_uri=https://selesnow.github.io/rfacebookstat/getToken/get_token.html&response_type=token&scope=",scopes))
    token <- readline(prompt = "Enter your token: ")
    options(rfacebookstat.access_token = token)
    return(token)
}

# checking token
fbCheckToken <- function(input_token, access_token) {
  
  link <- str_interp("https://graph.facebook.com/debug_token?input_token=${input_token}&access_token=${access_token}")
  
  ans <- GET(link)
  token_info <- content(ans)
  
  return(token_info$data)
  
}

# change token to lingtime
fbGetLongTimeToken <-
  function(client_id = NULL,client_secret = NULL,fb_exchange_token = NULL){
    if(is.null(client_id)|is.null(client_secret)|is.null(fb_exchange_token)){
      stop("Enter your Client ID, client_secret and short time token!")
    }
    link <- paste0("https://graph.facebook.com/oauth/access_token?grant_type=fb_exchange_token&client_id=",client_id,"&client_secret=",client_secret,"&fb_exchange_token=",fb_exchange_token)
    raw_token <- GET(link)
    token_list <- content(raw_token)
    
    if(!is.null(token_list$error)) {
      error <- token_list$error
      message(error)
    }
    
    message("Token changed to long time successfully")

    return(token_list$access_token)
}

# patj to dir with token file
fbTokenPath <- function() {
  
  
  ifelse( is.null(getOption("rfacebookstat.token_path")), 
          getwd(), 
          getOption("rfacebookstat.token_path") )
  
  
}

# user information
fbGetUserInfo <- function(access_token = getOption("rfacebookstat.access_token"),
                          user_id      = "me",
                          api_version  = getOption("rfacebookstat.api_version")) {
  
  link      <- paste0("https://graph.facebook.com/", api_version,"/", user_id, "?fields=id,name&access_token=", access_token)
  
  answer    <- GET(link)
  user_info <- content(answer)
  
  return(user_info)
  
}

# authorize
fbAuth <- function(username    = getOption("rfacebookstat.username"),
                   app_id      = getOption("rfacebookstat.app_id"), 
                   app_secret  = getOption("rfacebookstat.app_secret"), 
                   token_path  = fbTokenPath(),
                   scopes      =  c("ads_read", "business_management", "pages_manage_ads", "ads_management", "public_profile"),
                   reauth      = FALSE,
                   skip_option = FALSE) {
  
  access_path <- file.path(token_path, paste(username, "rfb_auth.rds", sep = "."))
  
  # check options
  if ( ! is.null(getOption( "rfacebookstat.access_token" )) && !skip_option ) {
    
    message("Access token was setted by option, if you want set new access token use skip_options = TRUE.")
    return( getOption( "rfacebookstat.access_token" ) )
    
  }
  
  # check environ variables
  if ( Sys.getenv("RFB_API_TOKEN") != "" && !skip_option ) {
    
    message("Access token was setted from environment variables, if you want set new access token use skip_options = TRUE.")
    return( Sys.getenv("RFB_API_TOKEN")  )
    
  }
  
  if ( file.exists( access_path ) && !reauth ) {
    
    fb_token <- readRDS(access_path)
    message(str_interp("Token load from ${access_path}"))
    
    if ( fb_token$token_info$expires_at != 0 ) {
      
      day_for_expired <- difftime(as.POSIXlt( fb_token$token_info$expires_at,  origin = "1970-01-01"), Sys.Date(), units = "days")
      
      if ( day_for_expired <= 10 ) {
        
        message(str_interp("Your token will be expires after ${day_for_expired} days, auto refreshing"))
        
        # change for longtime
        lt_token <- fbGetLongTimeToken(client_id         = app_id,
                                       client_secret     = app_secret, 
                                       fb_exchange_token = fb_token$access_token)
        
        # check token
        token_info <- fbCheckToken(lt_token, str_interp("${app_id}|${app_secret}"))
        
        # get user info 
        user_info  <- fbGetUserInfo(access_token = lt_token, 
                                    user_id      = ifelse( is.null(token_info$user_id), "me", token_info$user_id))
        
        # create token object
        fb_token   <- list(access_token = lt_token, 
                           token_info   = token_info,
                           user_info    = user_info)
        
        class(fb_token) <- "fb_access_token"
        
        # rewrite rds
        saveRDS(object = fb_token, 
                file   = access_path)
        
        message(str_interp("Refresh token saved in ${access_path}"))
        
      }
      
    }
    
    # set option
    options( rfacebookstat.access_token = fb_token$access_tokn)
    return(fb_token)
    
  } else {
    
    # check mode
    if ( !interactive() ) {
      stop(str_interp("File ${access_path} dont find, run fbAuth in interactive mode for authorization."))
    }
    
    # if reauth
    if ( reauth ) {
      message("Re authorization", ifelse( is.null(username), "", paste0(" under ",username)) )
    }
    
    # get shorttime token
    st_token <- fbGetToken(app_id, scopes)
    
    # change for longtime
    lt_token <- fbGetLongTimeToken(client_id         = app_id,
                                   client_secret     = app_secret, 
                                   fb_exchange_token = st_token)
    
    # check token
    token_info <- fbCheckToken(lt_token, str_interp("${app_id}|${app_secret}"))
    
    # get user info 
    user_info  <- fbGetUserInfo(access_token = lt_token, 
                                user_id      = token_info$user_id)
    
    # create token object
    fb_token   <- list(access_token = lt_token, 
                       token_info   = token_info,
                       user_info    = user_info)
    
    class(fb_token) <- "fb_access_token"
    
    # ask fo cache
    message(str_interp("Do you want save your access token into rds file ${access_path} for use it between R sessions ? "))
    to_cash <- readline( str_interp("y / n (recmedation y) ?: ") )
    
    # cache 
    if ( to_cash %in% tolower( c("y", "yes", "ok", "save") ) ) {
      
      if ( !dir.exists(token_path) ) {
        
        dir.create(token_path, recursive = T)
        
      }
      
      saveRDS(object = fb_token, 
              file   = access_path)
      
      message(str_interp("Token saved in ${access_path}"))
      
    }
    
    # set option
    options( rfacebookstat.access_token = fb_token$access_tokn ,
             rfacebookstat.username     = username)
    return(fb_token)
  }
}



print.fb_access_token <- function (x, show_token = FALSE, ...) {
  
  cat("\n", 
      "Facebook access token\n",
      "Access token: ", ifelse(show_token, x$access_token, "<hidden>"), "\n",
      "App id:       ", x$token_info$app_id, "\n",
      "App name:     ", x$token_info$application, "\n",
      "User id:      ", x$token_info$user_id, "\n",
      "User name:    ", x$user_info$name, "\n",
      "Expires at:   ", ifelse( x$token_info$expires_at == 0, "never", format(as.POSIXlt( x$token_info$expires_at,  origin = "1970-01-01"), "%Y-%m-%d %T"))
      )
	  
  if ( Sys.info()["sysname"] == "Windows" ) {
     utils::writeClipboard(x$access_token)
  }
 
}

# get settings
fbGetSettings <- function() {
  
  rfb_sets <- .Options[ grepl("^rfacebookstat", names(.Options) ) ]
  
  hidden_set <- c("rfacebookstat.app_secret", "rfacebookstat.access_token")
  
  
  
  message("You set ", length(rfb_sets), " options:")
  
  maxname <- max( nchar(names(rfb_sets) ) ) - 14
  
  for ( set in  names(rfb_sets) ) {
    
    key    <- gsub( "rfacebookstat\\.", "", set)
    spaces <- paste0(rep(" ", maxname - nchar(key)), collapse = "")
    key_space <- paste0(key, spaces, ":", collapse = "")
    value  <- ifelse( set %in% hidden_set, 
                      "<hidden>", 
                      as.character(
                          paste0(rfb_sets[[set]], collapse = ", ")
                          ) 
                      )
    value_length <- length(value)
    
    if ( value_length > 3 ) {
      value <- paste0("<vector of ", value_length, " values>", collapse = "")
    }
    
    cat(key_space, value, "\n")
    
  }
  
}

# revoke app privilegies
fbRevokeAppPrivilegies <- 
   function( app_id = getOption("rfacebookstat.app_id") ) {
  browseURL(str_interp("https://www.facebook.com/settings/?tab=business_tools&initial_open_app_id=${app_id}"))
}

# get all logins
fbGetLogins <- function(token_path = fbTokenPath(), set_login = TRUE) {

  files <- dir(token_path, pattern = "\\.rfb_auth\\.rds$")
  logins <- gsub("\\.rfb_auth\\.rds$", replacement = "", files)
  
  if (length(files) == 0) {
    message("You does't have any authorize login.")
    return(NULL)
  }
  
  for (i in 1:length(logins)) {
    cat(i, ":\t", logins[i], "\n")
  }
  
  if (set_login) {
    len <- length(logins)
    login_number <- readline(str_glue("Choose login (enter number from 1 to {len}): "))
    login <- logins[as.integer(login_number)]
    options(rfacebookstat.username = login)
    message(str_glue("Set {login} as default session login"))
    return(login)
  }
  
  return(logins)
  
}

fbGetCatalogs <- function(business_id  = getOption("rfacebookstat.business_id") ,
                          api_version  = getOption("rfacebookstat.api_version"),
                          username     = getOption("rfacebookstat.username"),
                          token_path   = fbTokenPath(),
                          access_token = getOption("rfacebookstat.access_token")) {
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  if(is.null(business_id)|is.null(access_token)){
    stop("Arguments business_id and access_token is require.")
  }
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  QueryString <- paste0("https://graph.facebook.com/",api_version,"/",business_id,"/owned_product_catalogs?limit=1500&access_token=",access_token)
  answer_owned_product_catalogs <- httr::GET(QueryString)
  
  # attr
  rq_ids      <- append(rq_ids, setNames(list(status_code(answer_owned_product_catalogs)), answer_owned_product_catalogs$headers$`x-fb-trace-id`))
  out_headers <- append(out_headers, setNames(list(headers(answer_owned_product_catalogs)), answer_owned_product_catalogs$headers$`x-fb-trace-id`))
  
  
  content_owned_product_catalogs <- content(answer_owned_product_catalogs)
  owned_product_catalogs <- map_df(content_owned_product_catalogs$data, ~ 
                                     data.frame(id = .x$id,
                                     name = .x$name)) %>%
                            mutate(catalog_typr = "owner")
 
  QueryString <- paste0("https://graph.facebook.com/",api_version,"/",business_id,"/client_product_catalogs?limit=1500&access_token=",access_token)
  answer_client_product_catalogs <- httr::GET(QueryString)
  
  # attr
  rq_ids      <- append(rq_ids, setNames(list(status_code(answer_client_product_catalogs)), answer_client_product_catalogs$headers$`x-fb-trace-id`))
  out_headers <- append(out_headers, setNames(list(headers(answer_client_product_catalogs)), answer_client_product_catalogs$headers$`x-fb-trace-id`))
    
  content_client_product_catalogs <- content(answer_client_product_catalogs)
  
  client_product_catalogs <- map_df(content_client_product_catalogs$data, ~ 
                                     data.frame(id = .x$id,
                                                name = .x$name)) %>%
                             mutate(catalog_typr = "client")
  
  result <- bind_rows(owned_product_catalogs, client_product_catalogs)
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  return(result)
}

fbGetAdVideos <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                          api_version  = getOption("rfacebookstat.api_version"),
                          username     = getOption("rfacebookstat.username"),
                          token_path   = fbTokenPath(),
                          access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
      access_token <- Sys.getenv("RFB_API_TOKEN")    
    } else {
      access_token <- fbAuth(username   = username, 
                             token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  res <- list()
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading videos from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  for( account_id in accounts_id ) {
    
    
    url <- str_interp("https://graph.facebook.com/${api_version}/${account_id}/advideos")
    
    api_answer  <- GET(url, 
                       query = list(fields       = "id,title,status,live_status,published,source,universal_video_id,length,permalink_url,description,from,icon,ad_breaks,backdated_time,backdated_time_granularity,content_category,created_time,updated_time,embeddable,embed_html,is_crosspost_video,is_crossposting_eligible,is_episode,is_instagram_eligible",
                                    limit        = 5000,
                                    access_token = access_token))
    
    # attr
    rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
    out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
    
    pars_answer <- content(api_answer, as = "parsed", encoding = "UTF-8")
    
    if(!is.null(pars_answer$error)) {
      error <- pars_answer$error
      message("\n",pars_answer$error,"\n")
    }
    
    # pars
    res <- append(res, 
                  lapply( pars_answer$data, fbParserAdVideos, account_id = account_id ))
    
    # paging
    while (!is.null(pars_answer$paging$`next`)) {
      
      api_answer  <- GET(pars_answer$paging$`next`)
      pars_answer <- content(api_answer, as = "parsed")
      
      # pars
      res <- append(res, 
                    lapply( pars_answer$data, fbParserAdVideos, account_id = account_id ))
      
    }
    
    # set progress bar
    if( pgbar ) {
      pb_step <- pb_step + 1
      setTxtProgressBar(pb, pb_step)
    }
  }
  
  res <- map_df(res, flatten)
  
  # set attributes
  attr(res, "request_ids") <- rq_ids
  attr(res, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(res)
}

fbGetAdAccountCustomAudiences <- 
  function(
    business_ids = getOption("rfacebookstat.business_id"),
    accounts_id  = getOption("rfacebookstat.accounts_id"),
    pixel_id     = NULL,
    filtering    = NULL,
    api_version  = getOption("rfacebookstat.api_version"),
    username     = getOption("rfacebookstat.username"),
    token_path   = fbTokenPath(),
    access_token = getOption("rfacebookstat.access_token")) {
    
    
    # auth 
    if ( is.null(access_token) ) {    
      
      if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
        access_token <- Sys.getenv("RFB_API_TOKEN")    
      } else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
      }
    }
    
    if ( inherits(access_token, "fb_access_token") ) {
      
      access_token <- access_token$access_token
      
    }
    
    
    # attributes
    rq_ids      <- list()
    out_headers <- list()
    
    res <- list()
    
    
    # load account list
    if ( is.null(accounts_id) ) {
      
      message("...Loading your account list.")
      accounts_id <- suppressMessages(fbGetAdAccounts()$id)
      message("...Loading customaudiences from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
      
    }
    
    # activate progress bar
    if ( length(accounts_id) > 1 ) {
      
      pgbar <- TRUE
      pb_step <- 1
      pb      <- txtProgressBar(pb_step, 
                                length(accounts_id), 
                                style = 3, 
                                title = "Loading:", 
                                label = "load" )
      
    } else {
      
      # progres bar
      pgbar <- FALSE
      
    }
    
    # filters handing to JSON
    if ( ! is.null(filtering) ) {
      if ( ! validate(filtering)[[1]] ) {
        
        filters <- list()
        
        for ( f in filtering ) {
          temp_fil    <- str_split(f, " ")
          temp_filter <- list(field    = temp_fil[[1]][1],
                              operator = temp_fil[[1]][2])
          
          if ( temp_fil[[1]][2] %in% c("IN_RANGE", "NOT_IN_RANGE", "IN", "NOT_IN") ) {
            temp_filter$value <- str_split(temp_fil[[1]][3], ",")[[1]]
          } else {
            temp_filter$value <- temp_fil[[1]][3]
          }
          filters <- append(filters, list(temp_filter))
        }
        filtering <- toJSON(filters, auto_unbox = T)
      }
    }
    
    for( account_id in accounts_id ) {
      
      
      url <- str_interp("https://graph.facebook.com/${api_version}/${account_id}/customaudiences")
      
      api_answer  <- GET(url, 
                         query = list(fields       = "id,account_id,approximate_count,customer_file_source,data_source,delivery_status,description,external_event_source,is_value_based,lookalike_audience_ids,lookalike_spec,name,operation_status,opt_out_link,permission_for_actions,pixel_id,retention_days,rule,rule_aggregation,sharing_status,subtype,time_content_updated,time_created,time_updated",
                                      filtering    = filtering,
                                      limit        = 30,
                                      access_token = access_token))
      
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
      
      pars_answer <- content(api_answer, as = "parsed")
      
      if(!is.null(pars_answer$error)) {
        error <- pars_answer$error
        if ( length(accounts_id) > 1 ) {
            warning(account_id, ' - ', pars_answer$error$message)
            next
        } else {
            stop("\n",pars_answer$error$message,"\n")
        }
      }
      
      # pars
      temp <- tibble(data = pars_answer$data) %>% 
              unnest_wider('data')
      
      if ( 'data_source' %in% names(temp) )            temp <- unnest_wider(temp, 'data_source', names_sep = "_")
      if ( 'delivery_status' %in% names(temp) )        temp <- unnest_wider(temp, 'delivery_status', names_sep = "_")
      if ( 'operation_status' %in% names(temp) )       temp <- unnest_wider(temp, 'operation_status', names_sep = "_")
      if ( 'permission_for_actions' %in% names(temp) ) temp <- unnest_wider(temp, 'permission_for_actions', names_sep = "_")
      if ( 'external_event_source' %in% names(temp) )  temp <- rowwise(temp) %>% mutate('external_event_source' = paste0(unlist('external_event_source'), collapse = ', '))
      if ( 'lookalike_audience_ids' %in% names(temp) ) temp <- rowwise(temp) %>% mutate('lookalike_audience_ids'  = paste0(unlist('lookalike_audience_ids'), collapse = ', '))
      if ( 'lookalike_spec' %in% names(temp) )         temp <- unnest_wider(temp, 'lookalike_spec', names_sep = "_") %>% unnest_wider('lookalike_spec_origin', names_sep = '_') %>% unnest_wider('lookalike_spec_origin_1', names_sep = "_")
      
      res <- append(res, 
                    list(temp))
      
      # paging
      while (!is.null(pars_answer$paging$`next`)) {
        
        api_answer  <- GET(pars_answer$paging$`next`)
        pars_answer <- content(api_answer, as = "parsed")
        
        if(!is.null(pars_answer$error)) {
          error <- pars_answer$error
          if ( length(accounts_id) > 1 ) {
            warning(account_id, ' - ', pars_answer$error$message)
            next
          } else {
            stop("\n",pars_answer$error$message,"\n")
          }
        }
        
        # pause between query
        Sys.sleep(0.2) 
        
        # pars
        temp <- tibble(data = pars_answer$data) %>% 
          unnest_wider('data')
        
        if ( 'data_source' %in% names(temp) )            temp <- unnest_wider(temp, 'data_source', names_sep = "_")
        if ( 'delivery_status' %in% names(temp) )        temp <- unnest_wider(temp, 'delivery_status', names_sep = "_")
        if ( 'operation_status' %in% names(temp) )       temp <- unnest_wider(temp, 'operation_status', names_sep = "_")
        if ( 'permission_for_actions' %in% names(temp) ) temp <- unnest_wider(temp, 'permission_for_actions', names_sep = "_")
        if ( 'external_event_source' %in% names(temp) )  temp <- rowwise(temp) %>% mutate('external_event_source' = paste0(unlist('external_event_source'), collapse = ', '))
        if ( 'lookalike_audience_ids' %in% names(temp) ) temp <- rowwise(temp) %>% mutate('lookalike_audience_ids'  = paste0(unlist('lookalike_audience_ids'), collapse = ', '))
        if ( 'lookalike_spec' %in% names(temp) )         temp <- unnest_wider(temp, 'lookalike_spec', names_sep = "_") %>% unnest_wider('lookalike_spec_origin', names_sep = '_') %>% unnest_wider('lookalike_spec_origin_1', names_sep = "_")
        
        res <- append(res, 
                      list(temp))
        
      }
      
      # pause between query
      Sys.sleep(0.2)  
      
      # set progress bar
      if( pgbar ) {
        pb_step <- pb_step + 1
        setTxtProgressBar(pb, pb_step)
      }
    }
    
    
    res <- bind_rows(res)
    
    # set attributes
    attr(res, "request_ids") <- rq_ids
    attr(res, "headers")     <- out_headers
    
    # close pb
    if( pgbar ) close(pb)
    
    return(res)
    
  }

fbGetCampaigns <- function(accounts_id  = getOption("rfacebookstat.accounts_id"),
                           api_version  = getOption("rfacebookstat.api_version"),
                           username     = getOption("rfacebookstat.username"),
                           token_path   = fbTokenPath(),
                           access_token = getOption("rfacebookstat.access_token")){
  
  # auth 
  if ( is.null(access_token) ) {    
    
    if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
	    access_token <- Sys.getenv("RFB_API_TOKEN")    
 	} else {
        access_token <- fbAuth(username   = username, 
                               token_path = token_path)$access_token
    }
  }
  
  if ( inherits(access_token, "fb_access_token") ) {
    
    access_token <- access_token$access_token
    
  }
  
  # load account list
  if ( is.null(accounts_id) ) {
    
    message("...Loading your account list.")
    accounts_id <- suppressMessages(fbGetAdAccounts()$id)
    message("...Loading campaigns from ", length(accounts_id), " account", ifelse( length(accounts_id) > 1, "s", "" ))
    
  }
  
  # check account format
  accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
  
  # attributes
  rq_ids      <- list()
  out_headers <- list()
  result <- list()
  
  # activate progress bar
  if ( length(accounts_id) > 1 ) {
    
    pgbar <- TRUE
    pb_step <- 1
    pb      <- txtProgressBar(pb_step, 
                              length(accounts_id), 
                              style = 3, 
                              title = "Loading:", 
                              label = "load" )
    
  } else {
    
    # progres bar
    pgbar <- FALSE
    
  }
  
  for( account_id in accounts_id ) {
  
      QueryString <- paste0("https://graph.facebook.com/",api_version,"/",account_id,"/campaigns?fields=id,name,created_time,bid_strategy,daily_budget,budget_remaining,spend_cap,buying_type,status,configured_status,account_id,recommendations,source_campaign_id,primary_attribution,campaign_attribution,start_time,stop_time&limit=1000",
                            "&filtering=[{'field':'campaign.delivery_info','operator':'NOT_IN','value':['stupid_filter']}]",
                            "&access_token=",access_token)
      
    
    
      api_answer  <- GET(QueryString)
      
      # attr
      rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
      out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
      
      pars_answer <- content(api_answer, as = "parsed")
      
      if(!is.null(pars_answer$error)) {
        error <- pars_answer$error
        stop(pars_answer$error)
      }
      
      result <- c(result, pars_answer$data)
      
      # paging
      while (!is.null(pars_answer$paging$`next`)) {
        
        api_answer  <- GET(pars_answer$paging$`next`)
    	
    	  # attr
        rq_ids      <- append(rq_ids, setNames(list(status_code(api_answer)), api_answer$headers$`x-fb-trace-id`))
        out_headers <- append(out_headers, setNames(list(headers(api_answer)), api_answer$headers$`x-fb-trace-id`))
    	
        pars_answer <- content(api_answer, as = "parsed")
        
        result <- c(result, pars_answer$data)
      }
      
      # set progress bar
      if( pgbar ) {
        pb_step <- pb_step + 1
        setTxtProgressBar(pb, pb_step)
      }
  }
  

  result <- bind_rows(result)
  
  # set attributes
  attr(result, "request_ids") <- rq_ids
  attr(result, "headers")     <- out_headers
  
  # close pb
  if( pgbar ) close(pb)
  
  return(result)
}

# limit check
print.fblimits <- function(x, ...) {
  cat(str_interp(" call_count: ${x[[1]]$call_count} \n total_cputime: ${x[[1]]$total_cputime} \n total_time: ${x[[1]]$total_time}"))
}


fbAPILimitCheck <- function( api_answer, console_type, pb, pb_step, accounts_id, pause_time ) {
  
  rep_query <- FALSE
  
  if ( "x-business-use-case-usage" %in% names(headers(api_answer)) ) {
    
    limitions <- fromJSON(headers(api_answer)$`x-business-use-case-usage`)
    
    class(limitions) <- "fblimits"
    
    if ( any( limitions[[1]]$call_count > 85, limitions[[1]]$total_cputime > 85, limitions[[1]]$total_time > 85 ) ) {
      
      if ( exists("console_type") ) {
        if (console_type == "progressbar"){
          close(pb)}
      }
      
      message("API Rate Limit Spent more than 85%")
      print(limitions)
      
      pause_time <- pause_time * 1.5
      
      if ( limitions[[1]]$estimated_time_to_regain_access > 0 ) {
        
        rep_query <- TRUE
      
        message("Wait for estimated_time_to_regain_access: ", limitions[[1]]$estimated_time_to_regain_access, " minutes")
        
        wait_minutes <- limitions[[1]]$estimated_time_to_regain_access
        
        wait_pb <- txtProgressBar(min = 0, max = wait_minutes, style = 3, title = "Please Waiting")
        
        for ( p in 1:wait_minutes ) {
          Sys.sleep( 60 )
          setTxtProgressBar(wait_pb, value = p, title = "Please Waiting")
        }
        Sys.sleep( 2 )
        close(wait_pb)
        
        if ( exists("console_type") ) {
          if (console_type == "progressbar"){
            pb <- txtProgressBar(pb_step, length(accounts_id), style = 3)
            utils::setTxtProgressBar(pb, pb_step)
          }
      }
    }
  }
}
  
  return(rep_query)
}

---
title: "1. API facebook Authorization"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{1. API facebook Authorization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(rfacebookstat)
```
Для начала работы с Facebook Marketing API предварительно требуется пройти процесс авторизации. В пакете `rfacebookstat` есть несколько вариантов авторизации:

* Простейшая авторизация с использованием приложения по умолчанию;
* Авторизация через собственное приложение.

# Простейшая авторизация
Наиболее быстрый и простой способ пройти авторизацию и получить маркер для работы с API Facebook - использовать приложение вшитое в пакет `rfacebookstat`. Для этого вам достаточно использовать функцию `fbAuth()`.

```{r}
library(rfacebookstat)

fbAuth()
```

После запуска функции `fbAuth()` вы будете перенаправлены в браузер для подтверждения разрешения пакету rfacebookstat доступа к вашим рекламным кабинетам. Далее вы будете перенаправлены на другую страницу, где для вас будет сгенерирован краткосрочный маркер доступа к API. Его необходимо скопировать и вставить в консоль RStudio в качестве ответа на запрос "Enter your token:".

Полученный ранее краткосрочный токен будет изменён на долгосрочный, о чём вы узнаете из сообщения "Token changed to long time successfully" в консоли RStudio.

У вас есть возможно сохранить полученные данные в файл, для того, что бы в следующий раз не было необходимости проходить авторизации через браузер. Для этого ответьте *y* или *yes* на вопрос "Do you want save your access token into rds file C:/my_develop_workshop/ppc_report_2/selesnow.rfb_auth.rds for use it between R sessions ?", который вы увидите в консоли RStudio.

Если вы сделали всё согласно инструкции, то вы увидите сообщение "Token saved in C:/facebook/credentials/.rfb_auth.rds", которое говорит о том, что вы успешно получили и сохранили учётные данные необходимые для работы с Facebook Narketing API. 

Также в консоль будет выведена некоторая информация о ваших учётных данных.

```
 Facebook access token
 Access token:  <hidden> 
 App id:        176943372709235 
 App name:      rfbstat 
 User id:       1246563312029308 
 User name:     Алексей Селезнёв 
 Expires at:    never
```

Из этого сообщения вы можете узнать id и название приложения которому вы предоставили разрешение на доступ к данным, id и имя пользователя под которым вы прошли авторизацию, а также дату до которой будет действителен полученный вами маркер доступа, never означает что вы получили бессрочный токен. 

В поле *Access token* должен отображаться ваш маркер, но по умолчанию он не выводится в консоль, если вы работаете под Windows, то каждый раз когда вы будете выводить на печать объект полученный с помощью функции `fbAuth()` ваш маркер доступа автоматически будет передан в буфер обмена.

Если вам необходимо вывести в консоль полученный маркер, то это можно сделать с помощью функции `print()` с использованием аргумента *show_token*.

```{r}
print(fbAuth(), show_token = T)
```
```
 Facebook access token
 Access token:  EAACg7dbg.................... 
 App id:        176943372709235 
 App name:      rfbstat 
 User id:       1246563312029308 
 User name:     Алексей Селезнёв 
 Expires at:    never
```

Теперь можно выполнить первый вызов к Facebook Marketing API. Например вы можете запросить список рекламным аккаунтам к которым у вас есть доступ с помощью функции `fbGetAdAccounts()`.

```{r}
fbGetAdAccounts()
```
```
Token load from C:/facebook/credentials/.rfb_auth.rds
# A tibble: 420 x 10
   id    name  account_id account_status amount_spent balance business_name currency owner
   <chr> <chr> <chr>               <int> <chr>        <chr>   <chr>         <chr>    <chr>
 1 act_~ 4772~ 47725506                1 9177567      2272    DEMOBAZA Ltd. EUR      7445~
 2 act_~ capp~ 14794670                1 2787098      0       ""            AED      2642~
 3 act_~ Rual~ 67398193                1 3681338      4825    Rual Travel ~ USD      2357~
 4 act_~ Plas~ 66869331                1 6355231      3069    Plasico Comp~ USD      8519~
 5 act_~ Maxi~ 77890760                1 2101480      426     Maxi.az       USD      1910~
 6 act_~ heal~ 171226248               1 3653879      0       Нетпик ЕООД   USD      3689~
 7 act_~ Tric~ 363293104               1 81285        0       ""            USD      5031~
 8 act_~ Spor~ 361373151               1 1424069      2053    Соларшоп ЕООД EUR      2201~
 9 act_~ Netp~ 262115113               1 7393426      2615    Netpeak       USD      9055~
10 act_~ Nata~ 362381897               1 6207085      6290    ""            USD      1950~
# ... with 410 more rows, and 1 more variable: user_role <chr>
```
Это действительно самый простой способ авторизации, но у API Facebook очень быстро меняются требования к приложениям в связи с чем, приложение которое вшито в `rfacebookstat` может быть недоступно для авторизации пользователям, если вы столкнулись с такой ошибкой то вы можете создать собственное приложение и пройти авторизацию через него. 

# Авторизация с использованием собственного приложения
Функция `fbAuth()` по умолчанию использует встроенное приложение, но при необходимости вы можете создать собственное приложение и пройти процесс авторизации через него.

## Создание и настройка собственного приложения для авторизации в Facebook API
Подробно о приложениях, их регистрации и настройке можно узнать в официальной [справке](https://developers.facebook.com/docs/development).

Для создания своего приложения необходимо выполнить следующие действия:

1. Перейти на [страниц приложений](https://developers.facebook.com/apps/).
2. Нажать кнопку "Добавьте новое приложение".
3. В появившемся диалоговом окне ввести название приложения и ваш email, и нажать "Создать ID приложения".
4. Далее в панели приложение добавьте продукт "Вход через Facebook", нажав на нём кнопку "Настроить"
5. Выберите из предложенных платформ "Веб".
6. В блоке "Укажите URL вашего сайта" введите https://selesnow.github.io, и нажмите "Save", а затем "Продолжить" > "Далее" > "Далее" > "Далее".
7. В меню вашего приложения в области "Продукты" вы увидите "Вход через Faceook", перейдите в раздел настройки.

![Настройки Вход через Facebook](http://img.netpeak.ua/alsey/157259948210_kiss_9kb.png)

8. В настройках найдите "Действительные URI перенаправления для OAuth" и введите следующий URL: https://selesnow.github.io/rfacebookstat/getToken/get_token.html
9. В нижней части экрана настройки "Вход через Facebook" нажмите "Сохранить изменения".
10. В основном меню приложения перейдите в раздел "Панель".
11. В окне "Панель" спуститесь в область "Добавьте продукты" и нажмите в продукте "API Marketing" кнопку "настроить".
12. Теперь можете вернуться в основном меню приложения в раздел Настройки > Основное.
13. В разделе Настройки > Основное вы найдёте Идентификатор приложения и секрет приложения, далее вы будете использовать эти параметры для авторизации через собственное приложение.

**Важно! Не выводите своё приложение из статуса "в разработке", т.к. для этого требуется проверка приложения со стороны поддержки API Facebook, процесс проверки длительный и сложный. Но вы можете работать со своими рекламными кампаниями даже если ваше приложение имеет статус "в разработке"**

## Аргументы функции `fbAuth()`

* app_id - Идентификатор приложения
* app_secret - Секрет приложения
* username - Ваш логин на Facebook
* token_path - Путь к папке в которой вы хотите создать файл для хранения учётных данных
* reauth - Переавторизоваться под указанным в username пользователем, если вы уже ранее запрашивали для него учётные данные 
* skip_option - Игнорировать опции и переменные окружения при авторизации (будет рассмотрено ниже)

Соответственно авторизоваться с помощью собственного приложения можно передав в аргументы *app_id* и *app_secret* идентификатор и секрет созданного вами приложения.

```{r}
fbAuth(app_id      = 556970798471513, 
       app_secret  = "10fbc64e0c426feb4e774395c97237fa",
       username    = "seleznev_a",
       skip_option = TRUE, 
       reauth      = FALSE,
       token_path  = "D:/fb_auth_store")
```
```
 Facebook access token
 Access token:  <hidden> 
 App id:        556970798471513 
 App name:      MyAPP 
 User id:       2834875706531386 
 User name:     Алексей Селезнёв 
 Expires at:    2019-12-31 09:32:15
```
При авторизации через собственное приложение с минимальным уровнем доступа к API срок действия вашего токена будет ограничен. В сообщение приведённом выше видно, что полученный токен действителен до 31 декабря 2019 года 09:32:15. 

На самом деле пакет `rfacebookstat` сам автоматически будет продлевать ваш токен по мере необходимости в случае если до завершена срока действия остаётся менее 10 дней.

# Автоматическая авторизация
Полученные вами учётные данные будут использоваться в каждом запросе к API Facebook. Поэтому после того, как вы один раз получили учётные данные, и сохранили их в файл, наиболее удобным вариантом их использования являются переменные среды или опции.

## Переменные среды
Это наиболее удобный способ работы с учётными данными в `rfacebookstat`, к тому же его преимущество заключается в том, что не будет необходимости хранить ваш токен в виде текстовой строки в скрипте.

Создать переменные среды можно несколькими способами:

1. Прописать их в файл *.Renviron*;
2. Настроить переменные окружения (для пользователей Windows):
3. Задать в начале скрипта с помощью команды `Sys.setenv()`.

### Настройка переменных среды через файл .Renviron
Файл *.Renviron* в домашнем каталоге R, и позволяет вам задавать переменные среды. 

Для начала необходимо найти домашний каталог:

```r
path.expand("~")
```
```
[1] "C:/Users/username/Documents"
```
Обычно это папка с вашими документами, как и в моём примере. Далее вам необходимо создать в этой папке файл *.Renviron*. И прописать в нём значение некоторых переменных, которые используются в `rfacebookstat`:

* RFB_TOKEN_PATH - Путь к папке в которой у вас хранится файл с раширением *.rfb_auth.rds*, в котором хранятся учётные данные.
* RFB_USER - Имя пользователя Facebook, который вы указали в аргументе *username* при прохождении авторизации с помощью функции `fbAuth()`.
* RFB_API_TOKEN - Полученный с помощью функции `fbAuth()` токен доступа к API.

На самом деле указывать все три переменные не имеет смысла, наибольший приоритет имеет переменная **RFB_API_TOKEN**. Если она указана то остальные две переменные игнорируются. Но использовать эту переменную имеет смысл только если вы получили бессрочный токен доступа, т.к. пакет не может обновить переменную среды, и если токен имеет срок действия по его истечению вы получите ошибку.

В случае если вы прошли авторизацию через собственное приложение, и получили токен с ограниченным сроком действия, следует использовать переменные **RFB_TOKEN_PATH** и **RFB_USER**. Указав имя пользователя и путь к папке в которую был сохранён файл с учётными данными в процессе авторизации через функцию `fbAuth()`, при авторизации вы можете указать папку с помощью аргумента *token_path*. По умолчанию файл сохраняется в рабочей директории на момент прохождения авторизации.

Т.е. у вы можете настроить файл *.Renviron* одним из двух вариантов.

```
RFB_API_TOKEN="abcdef788dsydsy9dcy"
```
Где `abcdef788dsydsy9dcy` ваш токен для работы с Facebook API.

Либо:
```
RFB_USER="seleznev_a"
RFB_TOKEN_PATH="D:/fb_auth_store"
```
### Настройка переменных среды в Windows
В Windows можно создать переменные среды следующем способом:

1. В строке "Поиск" выполните поиск: Система (Панель управления)
2. Нажмите на ссылку Дополнительные параметры системы.
3. Нажмите Переменные среды. 
4. Нажмите кнопку "Создать..."
5. Создайте нужные вам переменные, перечисленные в прошлом пункте.

![Переменная среды в Windows](http://img.netpeak.ua/alsey/157625864454_kiss_4kb.png)

### Задать в начале скрипта с помощью команды 
Ещё один способ, до подключения пакета `rfacebookstat` в скрипте задать переменные с помощью функции `Sys.setenv()`.

```r
Sys.setenv(RFB_USER="seleznev_a", 
           RFB_TOKEN_PATH="D:/fb_auth_store")

library(rfacebookstat)
```

### Как определить что вы успешно установили переменные среды
Если вы правильно определили переменные среды при подключении пакета в приветственном сообщении вы увидите отметку 'success' напротив успешно установленных переменных. Например если вы установили имя пользователя и путь к папке то вы увидите следующее сообщение:

```
rfacebookstat presets:
...Set rfacebookstat token_path: success
...Set rfacebookstat username: success
...Set rfacebookstat access_token: none
...Set Facebook Marketing API Version: v5.0
```
## Опции
Ещё один вариант инициализации учётных данных задать их с помощью опций. Также опции позволяют избежать дублирования аргументов без необходимости. Все опции вы можете задать в начале скрипта после подключения пакета.
В rfacebookstat доступны следующие опции:

* rfacebookstat.api_version - Версия API к которой пакет будет направлять запросы, не рекомендуется изменять эту опцию;
* rfacebookstat.access_token - Ваш токен доступа, также не рекомендуется хранить его текстом в ваших скриптах;
* rfacebookstat.accounts_id - ID аккаунтов которые вы используете в скрипте по умолчанию, можно задавать вектором;
* rfacebookstat.business_id - ID бизнес менеджера который вы планируете использовать в скрипте по умолчанию
* rfacebookstat.token_path - Путь к папке, где хранятся файлы с учётными данными;
* rfacebookstat.username - Имя пользователя facebook;
* rfacebookstat.app_id - ID созданного вами приложения в Facebook для авторизации;
* rfacebookstat.app_secret - Секрет созданного вами приложения в Facebook.

Пример использования опций:

```
library(rfacebookstat)

options(rfacebookstat.username = "seleznev_a",
        rfacebookstat.token_path = ""D:/fb_auth_store")

```

После установки опций каждая из функций пакета будет запрашивать значения большинства аргументов именно из опций, что избавит вам от излишнего дублирования этих значений в коде.

Так же для установки опций в пакете реализован набор функций с префиксом `fbSet*()`.

* `fbSetUsername(username)` - установка имя пользователя
* `fbSetAccount(accounts_ids)` -  установка идентификатор аккаунтов
* `fbSetBusinessId(business_ids)` - установка идентификаторов бизнес менеджеров
* `fbSetTokenPath(token_path)` - установка пути к папке для работы с токенами
* `fbSetApiVersion(api_version)` - установка версии API

## Приоритет переменных для авторизации

В каждом запросе к API необходимо передавать учётные данные, при этом каждая из функций пакета осуществляет поиск учётных данных по следующему пути.

1. Заданные в функции аргументы;
2. Опции;
3. Переменные среды.

Соответственно аргументы функции имеют максимальный приоритет.

# Рекомендации
Наиболее простой и правильный способ для работы с пакетом `rfacebookstat` использовать переменные среды для хранения имени пользователя и пути к папке с файлами в которых хранятся учётные данные.

С помощью опций устанавливать дефолтные значения для определения нужного бизнес менеджера и списка аккаунтов под каждый конкретный скрипт.

Данный подход избавит вас от избыточности и дублирование в коде.

## Получить список логинов под которыми была пройдена автоизация

С помощью функции `fbGetLogins()` вы можете запрашивать список логинов, под которыми вы уже успешно прошли авторизацию, и переключаться между ними.

---
title: "2. Get Facebook Marketing Insight"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{2. Get Facebook Marketing Insight}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)
```


Для загрузки статистических данных из рекламных аккаунтов Facebook в пакете `rfacebookstat` необходимо использовать функцию `fbGetMarketingStat`.

## Аргументы функции `fbGetMarketingStat`

Функция имеет множество аргументов с помощью которых вы можете максимально точно обозначить формат получаемых из API Facebook данных.

* *accounts_id* - ID рекламных аккаунтов, перед ID необходимо указать префикс `act_`, вы можете указать вектор из ID аккаунтов если требуется загрузить данные из набора аккаунтов. По умолчанию запрашивает значение опции *rfacebookstat.accounts_id*.
* *sorting* - Поле для сортировки полученных данных, так же вы можете указать направление добавив "_ascending" или "_descending" в поле сортировки. Например "reach_descending". Для действий вы можете сортировать по типу действия в форме "actions: <action_type>". Например, ["actions: link_click_ascending"]. Вы можете задать только одно поле для сортировки.
* *level* - Уровень группировки данных, принимает одно из следующих значений: ad, adset, campaign, account. По умолчанию имеет значение account.
* *action_breakdowns* - Разбивки по действиям, более подробно о них можно узнать в [документации](https://developers.facebook.com/docs/marketing-api/insights/breakdowns#actionsbreakdown) к API Facebook, так же более подробно они будут рассмотрены позже в этой виньетке.
* *breakdowns* - Общие разбивки которые предоставляют дополнительные возможности по группировке данных, подробности в [документации](https://developers.facebook.com/docs/marketing-api/insights/breakdowns#ageandgender) к API Facebook, так же этот аргумент будет более подробно рассмотрен немного ниже.
* *fields* - Поля которые вы хотите загрузить из API, полный список всех полей доступен по [ссылке](https://developers.facebook.com/docs/marketing-api/reference/ad-account/insights).
* *filtering* - Текстовая строка или вектор. Фильтрацию отчёта вы можете задавать либо в JSON формате либо в упрощенном формате, более подробно о фильтрации будет написано немного ниже.
* *date_start* - Дата начала отчётного периода.
* *date_stop* - Дата завершения рабочего отчётного периода.
* *date_preset* - С попомщью этого аргумента, вы можете использовать преднастройки для запроса данных за относительный период, например за прошлый месяц. Допустимые значения: today, yesterday, this_month, last_month, this_quarter, lifetime, last_3d, last_7d, last_14d, last_28d, last_30d, last_90d, last_week_mon_sun, last_week_sun_sat, last_quarter, last_year, this_week_mon_today, this_week_sun_today, this_year
* *request_speed* - Скорость отправки запросов, возможные значения "normal", "fast" или "slow". Так же можно задать числовое значение, которое будет задавать количество секунд в паузе между отправкой запросов к API.
* *fetch_by* - позволяет разбить большой запрос, за длительный период, на части по неделям, месяцам, кварталам и годам. Приннимает одно из следующих значений: "day", "week", "month", "quarter", "year".
* *api_version* - Версия API к которой будет отправлен запрос, по умолчанию берётся значение из опции *rfacebookstat.api_version*.
* *action_report_time* - Допустимые значения: impression, conversion. Определяет отчет о времени действия статистики. Например, если человек видел объявление 1 января, но совершил конверсию 2 января, при запросе API с `action_report_time = "impression"`, вы увидите конверсию 1 января. Когда вы запросите API с помощью `action_report_time = "conversion"`, вы увидите преобразование 2 января.
* *attribution_window* - Окна атрибуции
* *interval* - Группировка по временному интервалу, возможные значения: day, all_days, overall, monthly, weekly, либо вы можете числом указать количество дней, и результат вашего запроса будет сгруппирован такими временными отрезками.
* *use_unified_attribution_setting* - Если для этого параметра установлено значение true, результаты вашей рекламы будут показываться с использованием унифицированных настроек атрибуции, определенных на уровне группы объявлений, а параметр use_account_attribution_setting будет игнорироваться.
* *use_account_attribution_setting* - Если для этого параметра установлено значение true, результаты ваших объявлений будут отображаться с использованием настроек атрибуции, определенных для рекламного аккаунта.
* *console_type* - Тип выводимых в консоль данных, возможные значения progressbar, message.
* *username* - Логин на Facebook под которым вы проходили авторизацию, используется для хранения учётных данных для работы с API.
* *token_path* - Путь к папке в которой хранится файл с учётными данными на Facebook, которые вы получили при авторизации.
* *access_token* - Токен доступа к API полученный с помощью функции `fbGetToken`, по умолчанию запрашивается значение опции. *rfacebookstat.access_token*

## Пример запроса статистики

```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")

fb_data <- fbGetMarketingStat(
                  level              = "campaign",
                  fields             = "campaign_name,
                                        impressions,
                                        clicks,
                                        spend,
                                        actions",
                  date_start         = "2019-05-01",
                  date_stop          = "2019-05-10")
```

Выше приведён простейший способ загрузки статистики по показам, кликам, тратам и действиям в разрезе дней и рекламных кампаний.
Использование опций *rfacebookstat.access_token* и *rfacebookstat.accounts_id* не является обязательным, но поможет вам избежать дублирования этих параметров в аргументах всех функций пакета, поэтому я рекомендую именно такой способ установки значений accout_id, access_token, business_id и api_version.

## Учётные данные
С помощью аргументов *username* и *token_path* функция `fbGetMarketingStat()` определяет какие учётные данные ей необходимо использовать для запроса данных, и где эти учётные данные хранятся.

Более подробно об авторизации и этих аргументах написано в виньетке про авторизацию, открыть можно с помощью функции `vignette('rfacebookstat-authorization', package = 'rfacebookstat')`.

## Разбивки (breakdowns)
Разбивки помогают обогащать ваши данные за счёт дополнительных полей и группировок, на данный момент поддерживаются следующие разбивки:

* ad_format_asset
* age
* body_asset
* call_to_action_asset
* country
* description_asset
* gender
* image_asset
* impression_device
* link_url_asset
* product_id
* region
* title_asset
* video_asset
* dma
* frequency_value
* hourly_stats_aggregated_by_advertiser_time_zone
* hourly_stats_aggregated_by_audience_time_zone
* place_page_id
* publisher_platform
* platform_position
* device_platform

Актуальный список группировок всегда можно найти в официальной документации API Facebook по [ссылке](https://developers.facebook.com/docs/marketing-api/insights/breakdowns#ageandgender).

```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")

fb_data_breakdowns <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  breakdowns         = "region",
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

## Разбивки по действиям (action_breakdowns)
Вы можете сгруппировать результаты в поле *actions*. Аргумент *action_breakdowns* позволяет применить указанные ниже разбивки.

* action_device - Устройство, на котором произошло отслеживаемое событие конверсии. Например, «ПК», если человек выполнил конверсию с ПК.
* action_destination - Куда переходят люди, нажав вашу рекламу. Это может быть ваша Страница Facebook, внешний URL-адрес пикселя конверсий или приложение, сконфигурированное с помощью комплекта разработчика ПО.
* action_reaction - Количество реакций на ваши объявления или продвигаемые публикации. Кнопка реакций в рекламе позволяет людям по-разному отреагировать на ее содержание. Можно выбрать "Нравится", "Супер", "Ха-ха", "Ух ты!", "Сочувствую" или "Возмутительно".
* action_target_id - Идентификатор места назначения, в которое переходят люди, нажавшие на ссылку в вашей рекламе. Это может быть ваша Страница Facebook, внешний URL вашего пикселя конверсий, или приложение, сконфигурированное с помощью SDK.
* action_type - Тип действий, выполненных в отношении вашей рекламы, Страницы, приложения или мероприятия после показа объявления кому-либо, даже если эти люди не нажимали объявления. Типы действий включают отметки «Нравится» Страницы, установки приложений, конверсии, ответы на мероприятия и т.д.

```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")

fb_data_action_breakdowns <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  action_breakdowns  = "action_reaction",
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

### Описание полей возвращаемых в разбивке action_type
* app_custom_event.fb_mobile_achievement_unlocked: Разблокированные функции в мобильном приложении
* app_custom_event.fb_mobile_activate_app: Запуски мобильного приложения
* app_custom_event.fb_mobile_add_payment_info: Сведения об оплате в мобильном приложении
* app_custom_event.fb_mobile_add_to_cart: Добавления в корзину в мобильном приложении
* app_custom_event.fb_mobile_add_to_wishlist: Добавления в список желаний в мобильном приложении
* app_custom_event.fb_mobile_complete_registration: Регистрации в мобильном приложении
* app_custom_event.fb_mobile_content_view: Просмотры материалов мобильного приложения
* app_custom_event.fb_mobile_initiated_checkout: Оформление заказов в мобильном приложении
* app_custom_event.fb_mobile_level_achieved: Достижения в мобильном приложении
* app_custom_event.fb_mobile_purchase: Покупки в мобильном приложении
* app_custom_event.fb_mobile_rate: Оценки в мобильном приложении
* app_custom_event.fb_mobile_search: Поисковые запросы в мобильном приложении
* app_custom_event.fb_mobile_spent_credits: Траты кредитов в мобильном приложении
* app_custom_event.fb_mobile_tutorial_completion: Использования обучающей программы в мобильном приложении
* app_custom_event.other: Другие действия в мобильном приложении
* app_install: Установки приложения
* app_use: Использования приложения
* checkin: Посещения
* comment: Комментарии к публикации
* credit_spent: Траты кредитов
* games.plays: Количество раз, когда играли в вашу игру
* landing_page_view: Просмотры целевой страницы
* leadgen.other: Лиды (Форма)
* like: Отметки «Нравится» Страницы
* link_click: Количество кликов на ссылку
* mobile_app_install: Установки мобильного приложения
* offsite_conversion.custom.<custom_conv_id>: Индивидуально настроенные рекламодателем конверсии
* offsite_conversion.fb_pixel_add_payment_info: Добавление платежной информации
* offsite_conversion.fb_pixel_add_to_cart: Добавления товаров в корзину
* offsite_conversion.fb_pixel_add_to_wishlist: Добавления в список пожеланий
* offsite_conversion.fb_pixel_complete_registration: Завершенная регистрация
* offsite_conversion.fb_pixel_custom: Специально настроенные события пикселя, заданные рекламодателем
* offsite_conversion.fb_pixel_initiate_checkout: Инициирует оформление заказа
* offsite_conversion.fb_pixel_lead: Потенциальные клиенты
* offsite_conversion.fb_pixel_purchase: Покупки
* offsite_conversion.fb_pixel_search: Поиск
* offsite_conversion.fb_pixel_view_content: Просматривает контент
* onsite_conversion.flow_complete: Завершенные рабочие процессы на Facebook
* onsite_conversion.messaging_block: Заблокированные переписки
* onsite_conversion.messaging_conversation_started_7d: Начата переписка
* onsite_conversion.messaging_first_reply: Новые переписки
* onsite_conversion.post_save: Сохранения публикации
* onsite_conversion.purchase: Покупки на Facebook
* outbound_click: Исходящие клики
* photo_view: Отметки фотографии Страницы
* post: Репосты публикации
* post_reaction: Реакции на публикацию
* rsvp: Ответы на мероприятие
* video_view: 3-секундные просмотры видео

## Применить одновременно несколько разбивок
Некоторые разбивки можно применять одновременно.
Типы группирования, помеченные звездочкой (*), можно объединить с action_type, action_target_id и action_destination (название action_target_id).

* action_type *
* action_target_id *
* action_device *
* action_device, impression_device *
* action_device, publisher_platform *
* action_device, publisher_platform, impression_device *
* action_device, publisher_platform, platform_position *
* action_device, publisher_platform, platform_position, impression_device *
* action_reaction
* action_type, action_reaction
* age *
* gender *
* age, gender *
* country *
* region *
* publisher_platform *
* publisher_platform, impression_device *
* publisher_platform, platform_position *
* publisher_platform, platform_position, impression_device *
* product_id *

## Окна атрибуции
Количество дней между просмотром или нажатием рекламы и выполнением действия называется окном атрибуции. В функции `fbGetMarketingStat()` управление окнами атрибуции осуществляется с помощью аргумента *attribution_window*.

Действия с рекламой измеряются на основании кликов и просмотров:

* Атрибуция по кликам: человек нажал рекламу и выполнил действие.
* Атрибуция по просмотрам: человек посмотрел рекламу, не нажал ее, но выполнил действие в пределах окна атрибуции.

По умолчанию окно атрибуции настроено на 1 день после просмотра и 28 дней после клика. Это означает, что вы видите действия, которые произошли в течение 1 дня после просмотра рекламы и в течение 28 дней после ее нажатия. Вы можете изменить окно атрибуции на 1, 7 или 28 дней после просмотра или нажатия рекламы.

Чтобы посмотреть все действия, связанные с рекламой, вы можете настроить атрибуцию как после просмотра, так и после клика на один и тот же период времени.

Например, чтобы узнать, сколько покупок было совершено после просмотра или клика по рекламе за последние 7 дней, выберите для окон атрибуции по просмотрам и кликам продолжительность `attribution_window = c("7d_view", "7d_click")`. 

Аргумент *attribution_window* принимает вектор состоящий из окон атрибуции по которым вы хотите получить данные, при этом значение поля по каждой модели атрибуции которое вы получаете в массиве *actions* будет выделено в отдельный столбец. Например если вы применяете атрибуции *7d_view* и *7d_click*  то поле *lead* будет представлено в трёх столбцах: *lead*, *lead_7d_view*, *lead_7d_click*.

Возможные модели атрибуции: account_default, default, inline, 1d_view, 7d_view, 28d_view, 1d_click, 7d_click, 28d_click, 1d_view_1d_click, 7d_view_1d_click, 28d_view_1d_click, 28d_view_7d_click, 7d_view_7d_click, 28d_view_7d_click, 7d_view_28d_click, 28d_view_28d_click.

### Пример использования окон атрибуции


```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")

fb_data_action_breakdowns <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  action_breakdowns  = "action_reaction",
  attribution_window = c('default', '1d_view', '28d_view', '28d_click'),
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

Более подробно об окнах атрибуции в Facebook можно узнать по следующим ссылкам:

* [Об окнах атрибуции в Ads Manager](https://www.facebook.com/business/help/2198119873776795?id=768381033531365)
* [API Insights / Статистика / Окна атрибуции](https://developers.facebook.com/docs/marketing-api/insights/#sample)
* [Система правил для рекламы](https://developers.facebook.com/docs/marketing-api/ad-rules)

## Фильтрация данных
Вы можете применять фильтры к запрашиваемым данным. Использовать для этого необходимо аргумент `filtering`.
Указывать выражение для фильтрации ы можете в упрощённом формате или в виде JSON объектов, ниже я приведу пример использования обоих вариантов.

Для фильтрации вам необходимо указать поле по которому вы будете фильтровать данные, оператор и значение.
Допустимые операторы для фильтрации: EQUAL, NOT_EQUAL, GREATER_THAN, GREATER_THAN_OR_EQUAL, LESS_THAN, LESS_THAN_OR_EQUAL, IN_RANGE, NOT_IN_RANGE, CONTAIN, NOT_CONTAIN, IN, NOT_IN, STARTS_WITH, ANY, ALL, AFTER, BEFORE, NONE

### Пример фильтрации в упрощенном формате
```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")
fb_data <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  filtering          = "impressions LESS_THAN 5000",
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

В приведённом примере мы указали фильтр `impressions LESS_THAN 5000` и таким образом оставили строки в которых поле impressions имеет значение менее 5000.

Если вам необходимо использовать множественный оператор (IN_RANGE, NOT_IN_RANGE, IN, NOT_IN) то в упрошенном формате запись будет выглядеть так: `"publisher_platform IN instagram,facebook"`. Важно не ставить проблемы между списком значений.

```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")
fb_data <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  filtering          = "publisher_platform IN instagram,facebook",
  breakdowns         = "publisher_platform",
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

Если вы хотите применить несколько фильтров, то вы можете передать в аргумент `filtering` вектор из выражений, например: `c("clicks LESS_THAN 500", "impressions GREATER_THAN 1000")`.

```{r}
library(rfacebookstat)

options(rfacebookstat.access_token = "ваш токен",
        rfacebookstat.accounts_id  = "act_000000000")
fb_data <- fbGetMarketingStat(
  level              = "campaign",
  fields             = "campaign_name,
                        impressions,
                        clicks,
                        spend,
                        actions",
  filtering          = c("clicks LESS_THAN 500", "impressions GREATER_THAN 1000"),
  date_start         = "2019-05-01",
  date_stop          = "2019-05-10")
```

### Пример фильтрации в JSON формате
Как я уже писал выше вы можете описывать фильтры в виде JSON объектов, но такая запись будет более громоздка. Давайте  приведу вам аналогию с представленными выше фильтрами.

Упрощённый формат: `"impressions LESS_THAN 5000"`

JSON: `"[{"field":"impressions","operator":"LESS_THAN","value":"5000"}]"`

---

Упрощённый формат: `"publisher_platform IN instagram,facebook"`

JSON: `[{"field":"publisher_platform","operator":"IN","value":["instagram","facebook"]}]`

---

Упрощённый формат: `c("clicks LESS_THAN 500", "impressions GREATER_THAN 1000")`

JSON: `[{"field":"clicks","operator":"LESS_THAN","value":"500"},{"field":"impressions","operator":"GREATER_THAN","value":"1000"}]`

## Лимиты API и аргумент request_speed
По использованию аргумента `request_speed` есть целая [статья](https://alexeyseleznev.wordpress.com/2017/12/26/rfacebookstat-1-5-0-%D0%BA%D0%B0%D0%BA-%D0%BF%D1%80%D0%B0%D0%B2%D0%B8%D0%BB%D1%8C%D0%BD%D0%BE-%D0%B8%D1%81%D0%BF%D0%BE%D0%BB%D1%8C%D0%B7%D0%BE%D0%B2%D0%B0%D1%82%D1%8C-%D0%B0%D1%80%D0%B3%D1%83%D0%BC/), но я всё же немного опишу зачем данный аргумент нужен.

В API Facebook на данный момент существует 2 уровня доступа к API ([раздел в справке](https://developers.facebook.com/docs/marketing-api/get-started/authorization) API Facebook):

* Разработка - Тестирование приложений с помощью API.
* Стандартный уровень доступа для управления рекламой - Дополнительные ресурсы, например менее строгие ограничения количества обращений, а также возможность принять участие в программе для партнеров Facebook.

По умолчанию все создаваемые вами приложения получают уровень "Разработка". Данный уровень имеет серьёзные ограничения на количество отправляемых в API запросов. Функция `fbGetMarketingStat` при использовании аргумента `interval` равным `"day"` загружает данные по дням, и на каждый день отправляет отдельный запрос в API.

Так же отдельно разделяются запросы если вы загружаете данные сразу по несколько аккаунтам, таким образом если вы планируете загрузить данные с 1 января по 21 января по 3ём аккаунтам функция отправит (31 * 3) 93 запроса к API.

В случае если вы имеете стандартный доступ то для вас это не будет проблемой, и вы можете установить `request_speed = "fast"`, но для приложений с уровнем доступа "Разработка" такой объём отправляемых запросов может выйти далеко за лимиты API, от части `fbGetMarketingStat` умеет обходить такие лимиты каждый раз уходя в бан при их превышении, но скорость загрузки данных при попадании в бан будет очень низкий, иногда бан может составлять 5 минут.

Поэтому если ваше приложение имеет уровень доступа "Разработке" при загрузке данных по дням за длительный период рекомендуется использовать `request_speed = "slow"`. Если значение "slow" не помогает вы можете самостоятельно задавать паузу в секундах между запросами, например `request_speed = 4` будет задавать 4 секундную паузу между отправкой запросов.

Для получения стандартного доступа требуется перейти в [ваше приложение](https://developers.facebook.com/apps/) в раздел настроек API Marketing и отправить заявку на "Ads Management Standart Access".

---
title: "3. Loadind click and cost data from Facebook into Google Analytics"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{3. Loadind click and cost data from Facebook into Google Analytics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(rfacebookstat)
```


Начиная с версии `rfacebookstat 2.1.0` в пакет была добавлена функция `fbGetCostData()`, которая позволяет вам получить данные о расходах для дальнейшей их загрузки в Google Analytics.

## Создаём набор данных в Google Analytics
Изначально вам необходимо создать набор данных в Google Analytics.

1. Заходим в панель администратора
2. В области ресурс выбираем пункт "Импорт данных"

![Панель администратора](http://img.netpeak.ua/alsey/159056878065_kiss_43kb.png)

3. Нажимае **Создать / Create**
4. Среды предложенных вариантов выбираем **Данные о расходах / Cost Data**

![Выбираем тип набора данных](http://img.netpeak.ua/alsey/159056895184_kiss_45kb.png)

5. Задаём имя набора данных
6. Выбираем из выпадающего списка представления, в которые будут импортироваться данные о расодах

![Выбираем представления](http://img.netpeak.ua/alsey/159056902310_kiss_20kb.png)

7. Настраиваем схему данных и выбираем перезаписывать данные если они соответвуют уже существующим строкам *Owerwrite*

![Настраиваем схему данных](http://img.netpeak.ua/alsey/159056941892_kiss_43kb.png)

8. Жмём сохранить.

Далее в интерфейсе появится созданный набор данных.

![Созданный набор данных](http://img.netpeak.ua/alsey/159056960727_kiss_11kb.png)

В дальнейшем нам понадобится его идентификаторв.

## Запрашиваем данные о расхода из рекламного кабинета Facebook
Далее мы можем запросить данные о расходах из любого аккаунта к которому у нас есть доступ, и с помощью пакета `googleAnalyticsR` загрузить их в созданный источник данных.

```{r}
library(rfacebookstat)
library(googleAnalyticsR)

# fb options
options(rfacebookstat.username = 'your_facebook_username')

# auth
fbAuth()
ga_auth(email = "google_analytics@gmail.com")

# read data from facebook
cost_data <- fbGetCostData(accounts_id = 1111111,
                           date_start  = '2020-03-01',
                           date_stop   = '2020-03-10',
                           utm_source  = 'fb', 
                           utm_medium  = 'cpc')

# upload into GA source
ga_custom_upload_file(accountId          = 2222222, 
                      webPropertyId      = "UA-2222222-1", 
                      customDataSourceId = 'M_XXXXX_Qudk-8g', 
                      cost_data)
```

Если вы всё сделали правильно в разделе "Manage Uploads" вы увидите загруженные данные, и сообщение о статусе загрузке.

![Раздел "Manage Uploads"](https://img.netpeak.ua/alsey/159057045311_kiss_11kb.png)

![Статус загрузки](https://img.netpeak.ua/alsey/159057047222_kiss_16kb.png)

\name{fbGetPages}
\alias{fbGetPages}

\title{Get pages list}
\description{Load data about all avable pages}
\usage{
fbGetPages(accounts_id  = getOption("rfacebookstat.accounts_id"), 
           api_version  = getOption("rfacebookstat.api_version"), 
           username     = getOption("rfacebookstat.username"),
           token_path   = fbTokenPath(),
           access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{accounts_id}{Facebook Ad Account ID.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}


\value{Data frame with pages list}

\author{Alexey Seleznev}

\examples{
\dontrun{
accounts <- fbGetAccounts()
fbPages <- fbGetPages(accounts$id)
}
}

\name{fbGetAdAccountCustomAudiences}
\alias{fbGetAdAccountCustomAudiences}
\title{
The Custom Audiences Associated With the Ad Account.
}
\description{
Get The custom audiences associated with the ad account.
}
\usage{
fbGetAdAccountCustomAudiences(
    business_ids = getOption("rfacebookstat.business_id"),
    accounts_id  = getOption("rfacebookstat.accounts_id"),
    pixel_id     = NULL,
    filtering    = NULL,
    api_version  = getOption("rfacebookstat.api_version"),
    username     = getOption("rfacebookstat.username"),
    token_path   = fbTokenPath(),
    access_token = getOption("rfacebookstat.access_token")
)
}
\arguments{
  \item{business_ids}{ Your business manager or project ID.}
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{pixel_id}{This param fetches audiences associated to specific pixel.}
  \item{filtering}{Filters on the report data. This parameter is an array of filter objects.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}
\value{
data.frame with custom audiences data
}
\author{
Alexey Seleznev
}

\seealso{
 \href{https://developers.facebook.com/docs/marketing-api/reference/ad-account/customaudiences/}{Ad Account Customaudiences API Documentation}
}

\name{fbGetAdSets}
\alias{fbGetAdSets}

\title{Get creative list from facebook marketing API}
\description{Get ads creatives list with parameters from facebook marketing API. An ad set is a group of ads that share the same daily or lifetime budget, schedule, bid type, bid info, and targeting data. Ad sets enable you to group ads according to your criteria, and you can retrieve the ad-related statistics that apply to a set. }
\usage{
fbGetAdSets(accounts_id = getOption("rfacebookstat.accounts_id"), 
 api_version = getOption("rfacebookstat.api_version"), 
 username     = getOption("rfacebookstat.username"),
 token_path   = fbTokenPath(),
 access_token = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}

\value{
Data frame with ads list.
\item{1. id}{Ad set ID.}
\item{2. name}{Name of ad set.}
\item{3. account_id}{Ad Account ID.}
\item{4. budget_remaining}{Remaining budget.}
\item{5. configured_status}{The status set at the ad set level. It can be different from the effective status due to its parent campaign. Prefer using 'status' instead of this, one of ACTIVE, PAUSED, DELETED, ARCHIVED.}
\item{6. effective_status}{The effective status of the ad set, which can be either its own status or caused by its parent campaign, one of ACTIVE, PAUSED, DELETED, PENDING_REVIEW, DISAPPROVED, PREAPPROVED, PENDING_BILLING_INFO, CAMPAIGN_PAUSED, ARCHIVED, ADSET_PAUSED.}
\item{7. status}{The status set at the ad set level. It can be different from the effective status due to its parent campaign. The field returns the same value as 'configured_status', and is the suggested one to use (ACTIVE, PAUSED, DELETED, ARCHIVED).}
\item{8. created_time}{Created time.}
\item{9. bid_strategy}{Choose bid strategy for this ad set to suit your specific business goals. Each strategy has tradeoffs and may be available for certain optimization_goals:
LOWEST_COST_WITHOUT_CAP: Designed to get the most results for your budget based on your ad set optimization_goal without limiting your bid amount. This is the best strategy if you care most about cost efficiency. However with this strategy it may be harder to get stable average costs as you spend. This strategy is also known as automatic bidding. Learn more in Ads Help Center, About bid strategies: Lowest cost.
LOWEST_COST_WITH_BID_CAP: Designed to get the most results for your budget based on your ad set optimization_goal while limiting actual bid to your specified amount. With a bid cap you have more control over your cost per actual optimization event. However if you set a limit which is too low you may get less ads delivery. If you select this, you must provide a bid cap with the bid_amount field. Note: during creation this bid strategy is set if you provide bid_amount only. This strategy is also known as manual maximum-cost bidding. Learn more in Ads Help Center, About bid strategies: Lowest cost.
TARGET_COST: Maintains a stable average cost based on your optimization_goal as you raise your ad set budget. Select this strategy if you care most about maintaining stable average costs for your ads. However, note that this strategy may be less cost-efficient than the lowest cost strategy. If you select this, you must provide a target cost with the bid_amount field. Target cost bid strategy is also known as manual average-cost bidding. Learn more in Ads Help Center, About bid strategies: Target cost.
Note: If you enable campaign budget optimization, you should set bid_strategy at the parent campaign level.}
\item{10. pacing_type}{Defines the pacing type, standard by default or using ad scheduling}
}

\author{Alexey Seleznev}


\examples{
\dontrun{
library(rfacebookstat)
fb_token <- fbGetToken(app_id = 0000000)
adsets <- fbGetAdSets(accounts_id  = "act_11111111111111111",
                      api_version  = 'v3.0',
                      access_token = fb_token)
}

}

\name{fbGetCampaigns}
\alias{fbGetCampaigns}

\title{Get campaign list from facebook marketing API}
\description{Get campaign list with parameters from facebook marketing API.}
\usage{
fbGetCampaigns(accounts_id  = getOption("rfacebookstat.accounts_id"),
               api_version  = getOption("rfacebookstat.api_version"), 
			   username     = getOption("rfacebookstat.username"),
               token_path   = fbTokenPath(),
			   access_token = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{ Your account ID, for example 'act_11111111111111111'.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}

\value{
Data frame with campaign list.
}

\author{Alexey Seleznev}


\examples{
\dontrun{
library(rfacebookstat)
fb_token <- fbGetToken(app_id = 0000000)
camp <- fbGetCampaigns(accounts_id  = "act_11111111111111111",
                       api_version  = 'v3.0',
                       access_token = fb_token)
}
}

\name{fbGetBusinessManagersUsers}
\alias{fbGetBusinessManagersUsers}
\title{Get a list of business manager users}
\description{Loading user list from business managers.}
\usage{
fbGetBusinessManagersUsers(
   business_ids = getOption("rfacebookstat.business_id"),
   user_types   = c('business_users', 
                    'system_users', 
                    'pending_users'),
   api_version  = getOption("rfacebookstat.api_version"), 
   username     = getOption("rfacebookstat.username"),
   token_path   = fbTokenPath(),
   access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{business_ids}{IDs by your Business Manager}
  \item{user_types}{user types: business_users, system_users, pending_users.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with bussiness managers user list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
options(rfacebookstat.business_id  = 000000000000)

bm_users <- fbGetBusinessManagersUsers()
}
}

\name{fbGetAdAccountsConversions}
\alias{fbGetAdAccountsConversions}
\title{Get list of custom conversion from ad accounts.}
\description{Custom conversions on Facebook allows you to optimize and track actions without having to add anything to your Facebook pixel base code. They also allow you to optimize for and track actions that are different from the 9 standard events that come with the Facebook pixel.}
\usage{
fbGetAdAccountsConversions(
   accounts_id  = getOption("rfacebookstat.accounts_id"),
   api_version  = getOption("rfacebookstat.api_version"), 
   username     = getOption("rfacebookstat.username"),
   token_path   = fbTokenPath(),
   access_token = getOption("rfacebookstat.access_token")) 
}

\arguments{
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{access_token}{Your facebook API token.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with custom conversion list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
conversions <- fbGetAdAccountsConversions()
							  
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/authorization.R
\name{fbGetToken}
\alias{fbGetToken}
\title{Get API facebook token.}
\usage{
fbGetToken(
  app_id = NULL,
  scopes = c("ads_read", "business_management", "pages_manage_ads", "ads_management",
    "public_profile")
)
}
\arguments{
\item{app_id}{ID of your Facebook App}

\item{scopes}{Permissions provide a way for your app to access data from Facebook. For detail see \href{https://developers.facebook.com/docs/permissions}{docmentation}}
}
\value{
API token
}
\description{
Get API facebook token for access to facebook ads API.
}
\examples{
\dontrun{
tkn <- fbGetToken()
}
}

\name{fbGetApps}
\alias{fbGetApps}

\title{Get all the apps under a project}
\description{Get data frame with list apps under a project.}
\usage{
fbGetApps(accounts_id  = getOption("rfacebookstat.accounts_id"), 
         api_version  = getOption("rfacebookstat.api_version"), 
         username     = getOption("rfacebookstat.username"),
         token_path   = fbTokenPath(),
         access_token = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{Facebook Ad Account ID.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}

\value{
Data frame with apps list.
}

\author{Alexey Seleznev}


\examples{
\dontrun{
accounts <- fbGetAccounts()
fbApps <- fbGetApps(accounts$id)
}
}

\name{fbGetLogins}
\alias{fbGetLogins}
\title{
Get all authorization logins
}
\description{
Get all your authorized facebook logins
}
\usage{
fbGetLogins(
    token_path = fbTokenPath(), 
    set_login = TRUE)
}
\arguments{
  \item{token_path}{
path to dir with credentials
}
  \item{set_login}{
if TRUE you can choise and change current session login
}
}
\author{
Alexey Seleznev
}
\examples{
\dontrun{
fbGetLogins()
}
}

\name{fbGetAds}
\alias{fbGetAds}

\title{Get ads list from facebook marketing API}
\description{Get ads list with parameters from facebook marketing API. Contains information to display an ad and associate it an ad set. Each ad is associated with an ad set and all ads in a set have the same daily or lifetime budget, schedule, and targeting. Creating multiple ads in an ad set helps optimize their delivery based on variations in images, links, video, text or placements.}
\usage{
fbGetAds(
    accounts_id  = getOption("rfacebookstat.accounts_id"),
    fields       = c("id", 
                     "name" , 
                     "object_url",
                     "adlabels",
                     "adset_id",
                     "bid_amount",
                     "bid_type",
                     "campaign_id",
                     "account_id",
                     "configured_status",
                     "effective_status",
                     "creative"),
    api_version  = getOption("rfacebookstat.api_version"),
    username     = getOption("rfacebookstat.username"),
    token_path   = fbTokenPath(),
    access_token = getOption("rfacebookstat.access_token"),
    limit        = 5000)
}
\arguments{
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{fields}{Character vector of fields what you need, list of available fields <https://developers.facebook.com/docs/marketing-api/reference/adgroup/#---->.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
  \item{limit}{Row limit per 1 request.}
}

\value{
Data frame with ads list.
\item{1. id}{The ID of this ad.}
\item{2. name}{Name of the ad.}
\item{3. creative_id}{The ID or creative spec of the ad creative to be used by this ad.}
\item{4. adset_id}{The ID of the ad set, required on creation.}
\item{5. campaign_id}{ID of the ad campaign that contains this ad.}
\item{6. account_id}{The ID of the ad account that this ad belongs to.}
\item{7. bid_amount}{Bid amount for this ad which will be used in auction instead of the ad set bid_amount, if specified. Any updates to the ad set bid_amount will overwrite this value with the new ad set value.}
\item{8. bid_type}{Bid type, one of CPC, CPM, MULTI_PREMIUM, ABSOLUTE_OCPM, CPA}
\item{9. configured_status}{The configured status of the ad.}
\item{10. effective_status}{The effective status of the ad. The status could be effective either because of its own status, or the status of its parent units.}
}

\author{Alexey Seleznev}


\examples{
\dontrun{
library(rfacebookstat)
fb_token <- fbGetToken(app_id = 0000000)
ads <- fbGetAds(accounts_id  = "act_11111111111111111",
                api_version  = 'v3.0',
                access_token = fb_token)
}

}

\name{fbGetSettings}
\alias{fbGetSettings}

\title{Get settings.}
\description{Get settings of rfacebookstat package.}
\usage{
fbGetSettings()
}
\author{Alexey Seleznev}
\examples{
\dontrun{
fbGetSettings()
}
}

\name{fbGetUserAdAccounts}
\alias{fbGetUserAdAccounts}
\title{User's ad account list.}
\description{The advertising accounts to which this person has access.}
\usage{
fbGetUserAdAccounts(user_id      = "me", 
                    api_version  = getOption("rfacebookstat.api_version"), 
                    username     = getOption("rfacebookstat.username"),
                    token_path   = fbTokenPath(),
                    access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{user_id}{Facebook user id.}
  \item{access_token}{Your facebook API token.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
fbAccounts <- fbGetUserAdAccounts()
							  
}
}

\name{fbGetAdAccountUsersPermissions}
\alias{fbGetAdAccountUsersPermissions}
\title{Get ad account user list with him permissions.}
\description{fbGetAdAccountUsersPermissions get ad account user list with him permissions. in R.}
\usage{
fbGetAdAccountUsersPermissions(
    accounts_id = getOption("rfacebookstat.accounts_id"), 
	api_version = getOption("rfacebookstat.api_version"), 
	console_type = "progressbar", 
	username     = getOption("rfacebookstat.username"),
	token_path   = fbTokenPath(),
	access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{accounts_id}{vector with ID of your ad account, for example c("act_00001","act_00002").}
  \item{access_token}{Your facebook API token.}
  \item{console_type}{Console output type.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account user permissions list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
UP <- fbGetAdAccountUsersPermissions(accounts_id = c("act_00001","act_00002"), 
                                     api_version = "v2.10", 
	    						     access_token = "xxxxxx")
}
}

\name{fbAuth}
\alias{fbAuth}

\title{Authorization in Facebook.}
\description{Wrapper for \code{fbGetToken()} and \code{fbGetLongtimeToken()}.}
\usage{
fbAuth(username    = getOption("rfacebookstat.username"),
       app_id      = getOption("rfacebookstat.app_id"), 
       app_secret  = getOption("rfacebookstat.app_secret"), 
       token_path  = fbTokenPath(),
       scopes      = c("ads_read", 
                       "business_management", 
                       "pages_manage_ads", 
                       "ads_management", 
                       "public_profile"),
	     reauth      = FALSE,
       skip_option = FALSE)
}

\arguments{
  \item{username}{your username on Facebook}
  \item{app_id}{ID of Facebook App}
  \item{app_secret}{Secret of Facebook App}
  \item{token_path}{path to dir with credentials}
  \item{scopes}{Permissions provide a way for your app to access data from Facebook. For detail see \href{https://developers.facebook.com/docs/permissions}{docmentation}}
  \item{reauth}{boolean, do reauthorization}
  \item{skip_option}{logical, skip access token from options or not}
}

\value{API token object}

\author{Alexey Seleznev}

\examples{
\dontrun{
fbAuth()
}
}

\name{rfacebookstat-package}
\alias{rfacebookstat-package}
\alias{rfacebookstat}
\docType{package}
\title{
\packageTitle{rfacebookstat}
}
\description{
\packageDescription{rfacebookstat}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{rfacebookstat}
\packageIndices{rfacebookstat}
}
\author{
\packageAuthor{rfacebookstat}

Maintainer: \packageMaintainer{rfacebookstat}
}

\keyword{ package }
\seealso{
See oficial documantation:
rfacebookstat package - https://selesnow.github.io/rfacebookstat/
Facebook API Marketing - https://developers.facebook.com/docs/marketing-api
}
\examples{
\dontrun{
library(rfacebookstat)
# Auth
token <- fbGetToken(app_id = 00000000000000)

# Get statistic
CampStat <- fbGetMarketingStat(accounts_id = "act_0000000000",
level = "campaign", fields = "campaign_name,impressions,clicks",
breakdowns = "age", sorting = "unique_impressions_descending",
filtering = "[{'field':'age','operator':'IN','value':['18-24','25-34']}]",
date_start = "2016-10-01", date_stop = "2016-10-10",
access_token = token)
}
}

\name{fbGetAdVideos}
\alias{fbGetAdVideos}

\title{Get videos list from ad accounts}
\description{Load all videos from faceboob ad accounts.}
\usage{
fbGetAdVideos(accounts_id  = getOption("rfacebookstat.accounts_id"), 
              api_version  = getOption("rfacebookstat.api_version"),
		      username     = getOption("rfacebookstat.username"),
	          token_path   = fbTokenPath(),
              access_token = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{api_version}{Current Facebook API version.}
  \item{username}{Your username on Facebook}
  \item{token_path}{Path to dir with credentials}
  \item{access_token}{Your facebook API token.}
}

\value{
Data frame with video parameters list.
}

\author{Alexey Seleznev}


\examples{
\dontrun{
ads <- fbGetAdVideos()
}

}

\name{fbSetters}
\alias{fbSetters}
\alias{fbSetUsername}
\alias{fbSetAccount}
\alias{fbSetBusinessId}
\alias{fbSetTokenPath}
\alias{fbSetApiVersion}
\title{
Set rfacebookstat options
}
\description{
Set rfacebookstat options in current R session
}
\usage{
fbSetUsername(username)
fbSetAccount(accounts_ids) 
fbSetBusinessId(business_ids)
fbSetTokenPath(token_path)
fbSetApiVersion(api_version)
}
\arguments{
  \item{accounts_ids}{Facebook Ad Account ID.}
  \item{business_ids}{IDs by your Business Manager}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}
\author{
Alexey Seleznev
}

\name{fbUpdateAdAccountUsers}
\alias{fbUpdateAdAccountUsers}
\title{Add users and update permission list.}
\description{fbUpdateAdAccountUsers add users and update permission list for users in ad accounts at facebook.}
\usage{
fbUpdateAdAccountUsers(
   user_ids     = NULL, 
   role         = "advertiser", 
   accounts_id  = getOption("rfacebookstat.accounts_id"), 
   api_version  = getOption("rfacebookstat.api_version"), 
   username     = getOption("rfacebookstat.username"),
   token_path   = fbTokenPath(),
   access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{accounts_id}{vector with ID of your ad account, for example c("act_00001","act_00002").}
  \item{role}{Character or integer value, one of administator, advertiser, ad manager, analyst, sales, direct sales, 1001, 1002, 1003,1004 (https://developers.facebook.com/docs/marketing-api/aduser/v2.12).}
  \item{access_token}{Your facebook API token.}
  \item{user_ids}{vector with users ids who need remove from ad account.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\details{
Requires:
ads_management permission is required.
You need 'Administrator' access to the ad account to remove users

Roles:
1001 = Administrator access
1002 = Advertiser, or ad manager, access
1003 = Analyst access
1004 = Direct sales access. For limited, directly managed accounts.

Permissions:
1: ACCOUNT_ADMIN: modify list of people associated with the account.
2: ADMANAGER_READ: view campaigns and ads
3: ADMANAGER_WRITE: manage campaigns and ads
4: BILLING_READ: view account billing information
5: BILLING_WRITE: modify account billing information
7: REPORTS: run reports
9, 10 - Apply to some directly managed accounts. Not currently needed for Marketing API calls

For more details go link (https://developers.facebook.com/docs/marketing-api/aduser/)
}
\author{Alexey Seleznev}

\examples{
\dontrun{
#Attach package
library(rfacebookstat)

#Get token, where 0000000000000 is id of your app in facebook
tk <- fbGetToken(0000000000000) 

#Add user with id 001 and 002 to accounts act_0001 and act_0002
fbUpdateAdAccountUsers(user_ids = c("001","002"), 
                       accounts_id = c("act_0001","act_0002"), 
                       role = "advertiser",
                       api_version = "v2.12",
                       access_token = tk)
}
}

\name{fbGetCostData}
\alias{fbGetCostData}

\title{Get facebook ads cost data.}
\description{Get cost data for loading it into Google Analytics}
\usage{
fbGetCostData(
    accounts_id = getOption("rfacebookstat.accounts_id"),
    date_start         = Sys.Date() - 30,
    date_stop          = Sys.Date(),
    utm_source         = "facebook",
    utm_medium         = "cpc",
    username           = getOption("rfacebookstat.username"),
    token_path         = fbTokenPath(),
    access_token       = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{ID of your ad account.}
  \item{date_start}{Start reporting date.}
  \item{date_stop}{End reporting day}
  \item{utm_source}{UTM source name for Google Analytics.}
  \item{utm_medium}{UTM medium name for Google Analytics.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token}
}

\value{Data frame with statistic.}
\details{
Before loading data create data source at Google Analytics.
}


\author{Alexey Seleznev}

\examples{
\dontrun{

# Use googleAnalyticsR for upload data into Google Analytics
library(googleAnalyticsR)

# load cost data
cost_data <- fbGetCostData(accounts_id = "act_11111",
                           date_start  = "2020-05-10", 
                           date_stop   = "2020-05-14",
                           username    = "fb_login")
                              
# upload into GA source
ga_custom_upload_file(accountId          = xxxx, 
                      webPropertyId      = "UA-xxxx-1", 
                      customDataSourceId = 'abcdefg', 
                      cost_data)
}
}

\name{fbGetMarketingStat}
\alias{fbGetMarketingStat}

\title{Get statistic by ad accounts.}
\description{fbGetMarketingStat is main function of rfacebookstat package, intended for load statiscit data by your ad, adset, campaign or account}
\usage{
fbGetMarketingStat(
  accounts_id = getOption("rfacebookstat.accounts_id"), 
  sorting = NULL, level = "account", breakdowns = NULL, 
	action_breakdowns = NULL, 
	fields = "account_id,campaign_name,impressions,clicks,reach,spend",
  filtering = NULL, date_start = NULL,
  date_stop = NULL, date_preset = "last_30d",
  attribution_window = NULL,
	api_version = getOption("rfacebookstat.api_version"), 
	action_report_time = NULL, interval = "day", 
	use_unified_attribution_setting = FALSE,
	use_account_attribution_setting = FALSE,
	console_type = "progressbar", request_speed = "normal",
	fetch_by = NULL,
  username = getOption("rfacebookstat.username"),
  token_path = fbTokenPath(),	
	access_token = getOption("rfacebookstat.access_token"))
}
\arguments{
  \item{accounts_id}{ID of your ad account.}
  \item{sorting}{Field to sort the result, and direction of sorting. You can specify sorting direction by appending "_ascending" or "_descending" to the sort field. For example, "reach_descending". This array supports no more than one element. By default, the sorting direction is ascending.}
  \item{level}{Represents the level of result. Avable ad, adset, campaign, account.}
  \item{action_breakdowns}{group results in the actions field. You can use the following breakdowns for action_breakdowns, for details \href{https://developers.facebook.com/docs/marketing-api/insights/breakdowns/#actionsbreakdown}{go}, or see details}
  \item{breakdowns}{Permutations marked with an asterisk (*) can be joined with action_type and action_target_id. Avable age, country, gender, frequency_value, hourly_stats_aggregated_by_advertiser_time_zone, hourly_stats_aggregated_by_audience_time_zone, impression_device, place_page_id, placement, device_platform, product_id, region. See details.}
  \item{fields}{List of fields which you want get in R.}
  \item{filtering}{Vector of filtring or JSON string with array of filtring parameters, on example \code{"ad.effective_status IN ARCHIVED"} \code{[{/'field/':/'ad.effective_status/',/'operator/':/'IN/',/'value/':[/'ARCHIVED/']}}, see filtring block for more examples}
  \item{date_start}{Start reporting date.}
  \item{date_stop}{End reporting day}
  \item{date_preset}{Represents a relative time range. This field is ignored if time_range or time_ranges is specified. One of: today, yesterday, this_month, last_month, this_quarter, lifetime, last_3d, last_7d, last_14d, last_28d, last_30d, last_90d, last_week_mon_sun, last_week_sun_sat, last_quarter, last_year, this_week_mon_today, this_week_sun_today, this_year}
  \item{attribution_window}{The conversion attribution window provides timeframes that define when we attribute an event to an ad on Facebook. See Attribution Window sections.}
  \item{request_speed}{Speed beetwen API request, "normal", "fast" or "slow", depend of you API access level.}
  \item{fetch_by}{Character, split your requst by time interval, one of: day, week, month, quarter, year.}
  \item{api_version}{Current Facebook API version.}
  \item{action_report_time}{Determines the report time of action stats.}
  \item{interval}{Character value for split by time interval, one of "day", "week", "month", "quarter", "year". By default = "day"}
  \item{use_unified_attribution_setting}{When this parameter is set to true, your ads results will be shown using unified attribution settings defined at ad set level and parameter use_account_attribution_setting will be ignored.}
  \item{use_account_attribution_setting}{When this parameter is set to true, your ads results will be shown using the attribution settings defined for the ad account.}
  \item{console_type}{Character value for manage console output message, one of "progressbar", "message". By default = "progressbar". See more at Detail}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token}
}

\value{Data frame with statistic.}
\details{
Console_type parameters, if chose "progressbar" you can see load progress in percent, and if chose "message" you get message about loading process.
}

\section{Breakdowns}{
Allowed values for breakdowns.
\itemize{
  \item ad_format_asset
  \item age
  \item body_asset
  \item call_to_action_asset
  \item country
  \item description_asset
  \item gender
  \item image_asset
  \item impression_device
  \item link_url_asset
  \item product_id
  \item region
  \item title_asset
  \item video_asset
  \item dma
  \item frequency_value
  \item hourly_stats_aggregated_by_advertiser_time_zone
  \item hourly_stats_aggregated_by_audience_time_zone
  \item place_page_id
  \item publisher_platform
  \item platform_position
  \item device_platform
 }
}

\section{Action Breakdowns}{
Group results in the actions field. You can use the following breakdowns for action_breakdowns.
Now you can use next action breakdowns:
\describe{
  \item{action_device}{The device on which the conversion event you're tracking occurred. For example, "Desktop" if someone converted on a desktop computer.}
  \item{action_destination}{The destination where people go after clicking on your ad. This could be your Facebook Page, an external URL for your conversion pixel or an app configured with the software development kit (SDK).}
  \item{action_reaction}{The number of reactions on your ads or boosted posts. The reactions button on an ad allows people to share different reactions on its content: Like, Love, Haha, Wow, Sad or Angry.}
  \item{action_target_id}{The id of destination where people go after clicking on your ad. This could be your Facebook Page, an external URL for your conversion pixel or an app configured with the software development kit (SDK).}
  \item{action_type}{The kind of actions taken on your ad, Page, app or event after your ad was served to someone, even if they didn't click on it. Action types include Page likes, app installs, conversions, event responses and more.}
  \item{action_type,action_reaction}{Together of action types and reactions brekdown.}
 }
}

\section{Available Combinations Of Breakdowns}{
Grouping types marked with an asterisk (*) can be combined with action_type, action_target_id, and action_destination (action_target_id).
\itemize{
  \item action_type *
  \item action_target_id *
  \item action_device *
  \item action_device, impression_device *
  \item action_device, publisher_platform *
  \item action_device, publisher_platform, impression_device *
  \item action_device, publisher_platform, platform_position *
  \item action_device, publisher_platform, platform_position, impression_device *
  \item action_reaction
  \item action_type, action_reaction
  \item age *
  \item gender *
  \item age, gender *
  \item country *
  \item region *
  \item publisher_platform *
  \item publisher_platform, impression_device *
  \item publisher_platform, platform_position *
  \item publisher_platform, platform_position, impression_device *
  \item product_id *
}
}

\section{Attribution Window}{
The conversion attribution window provides time intervals that determine the attribution period of an event for advertising on Facebook.
For background information, see Facebook Ads Help Center, How Attribution Reporting Works. We measure the actions that occur when a conversion event occurs and look back in time 1-day, 7-days, and 28 days.
To view actions attributed to different attribution windows, use \code{attribution_window}.

\describe{
  \item{account_default}{Use the account level attribution window setting}
  \item{default}{The FB default attribution window is 1 day views, 28 day clicks}
  \item{inline}{Inline attribution only (0 day views, 0 day clicks)}
  \item{1d_view}{1 day views, 0 day clicks}
  \item{7d_view}{7 day views, 0 day clicks}
  \item{28d_view}{28 day views, 0 day clicks}
  \item{1d_click}{0 day views, 1 day clicks}
  \item{7d_click}{0 day views, 7 day clicks}
  \item{28d_click}{0 day views, 28 day clicks}
  \item{1d_view_1d_click}{1 day views, 1 day clicks}
  \item{7d_view_1d_click}{7 day views, 1 day clicks}
  \item{28d_view_1d_click}{28 day views, 1 day clicks}
  \item{1d_view_7d_click}{1 day views, 7 day clicks}
  \item{1d_view_28d_click}{1 day views, 28 day clicks}
  \item{28d_view_28d_click}{28 day views, 28 day clicks}
 }

Also you can get more than one attribution window in one request, for example \code{attribution_window = c('default', '1d_view', '28d_view', '28d_click')}
}

\section{Filtering}{
Filters on the report data. This parameter is an array of filter objects.
You can set string vector or JSON string with field, operator and value.
\describe{
  \item{field}{Field for filtering.}
  \item{operator}{One of EQUAL, NOT_EQUAL, GREATER_THAN, GREATER_THAN_OR_EQUAL, LESS_THAN, LESS_THAN_OR_EQUAL, IN_RANGE, NOT_IN_RANGE, CONTAIN, NOT_CONTAIN, IN, NOT_IN, STARTS_WITH, ANY, ALL, AFTER, BEFORE, NONE.}
  \item{value}{Field value for filtering.}
 }
 Simple filtring:
 \describe{
   \item{Example vector}{\code{filtering = "publisher_platform IN instagram"}}
   \item{Example JSON}{\code{filtering = "[{'field':'publisher_platform','operator':'IN','value':['instagram']}]"}}
 Example of using two or more conditions: \code{filtering = "[{'field':'clicks','operator':'LESS_THAN','value':500},{'field':'impressions','operator':'GREATER_THAN','value':20000}]"}
}
Filtring by two or more conditions:
\describe{
   \item{Example vector}{\code{filtering = c("clicks LESS_THAN 500", "impressions GREATER_THAN 20000")}}
   \item{Example JSON}{\code{filtering = '[{"field":"clicks","operator":"LESS_THAN","value":"500"},{"field":"impressions","operator":"GREATER_THAN","value":"20000"}]'}}
   }
Filtring with operators IN_RANGE, NOT_IN_RANGE, IN, NOT_IN

\describe{
   \item{Example vector}{\code{filtering = 'publisher_platform IN instagram,facebook'}}
   \item{Example JSON}{\code{filtering = '[{"field":"publisher_platform","operator":"IN","value":["instagram","facebook"]}]'}}
   }
}
\author{Alexey Seleznev}

\seealso{
All avable field, and discription of all parameters you can see oh this  \href{https://developers.facebook.com/docs/marketing-api/reference/ad-account/insights}{link}.
More information about breakdowns and action brekdowns on this \href{https://developers.facebook.com/docs/marketing-api/insights/breakdowns}{link}.
More information about action field in this \href{https://developers.facebook.com/docs/marketing-api/reference/ads-action-stats/}{documentation}.
}
\examples{
\dontrun{
fbStat <- fbGetMarketingStat(accounts_id = "act_xxxxxxxxxxxxxxx",
                             level = "campaign",
                             fields = "account_name,campaign_name,impressions",
                             breakdowns = "device_platform",
                             date_start = "2016-08-01",
                             date_stop = "2016-08-10",
			     interval = "day",
                             access_token = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
}
}

\name{fbGetAdAccounts}
\alias{fbGetAdAccounts}
\title{Get avable ad account list.}
\description{fbGetAdAccounts get data frame with avable in your bussines manager ad account list in R.}
\usage{
fbGetAdAccounts(source_id    = getOption("rfacebookstat.business_id"), 
                api_version  = getOption("rfacebookstat.api_version"), 
                username     = getOption("rfacebookstat.username"),
                token_path   = fbTokenPath(),
				access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{source_id}{ Your business manager or project ID.}
  \item{access_token}{Your facebook API token.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
fbAccounts <- fbGetAdAccounts(source_id = "xxxxxxxxxxxxx",
                              api_version = "v2.10", 
							  access_token = "xxxxxxxx")
							  
}
}

\name{fbDeleteAdAccountUsers}
\alias{fbDeleteAdAccountUsers}
\title{Delete users from ad accounts.}
\description{fbDeleteAdAccountUsers is method for remove users access from ad accounts.}
\usage{
fbDeleteAdAccountUsers(
    user_ids     = NULL, 
    accounts_id  = getOption("rfacebookstat.accounts_id"),
	api_version  = getOption("rfacebookstat.api_version"),
	username     = getOption("rfacebookstat.username"),
    token_path   = fbTokenPath(),
	access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{accounts_id}{vector with ID of your ad account, for example c("act_00001","act_00002").}
  \item{access_token}{Your facebook API token.}
  \item{user_ids}{vector with users ids who need remove from ad account.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\details{
ads_management permission is required.
You need 'Administrator' access to the ad account to remove users
}
\author{Alexey Seleznev}

\examples{
\dontrun{
#Attach package
library(rfacebookstat)
#Get token, where 0000000000000 is id of your app in facebook
tk <- fbAuth(0000000000000) 
#Remove user with id 001 and 002 from accounts act_0001 and act_0002
fbDeleteAdAccountUsers(user_ids = c(001,002), 
                       accounts_id = c("act_0001","act_0002"), 
                       api_version = "v2.12",
                       access_token = "abcdef123456" )
}
}

\name{fbGetCatalogs}
\alias{fbGetCatalogs}
\title{Get catalogs}
\description{Load catalogs from business maneger.}
\usage{
fbGetCatalogs(business_id  = getOption("rfacebookstat.business_id") ,
              api_version  = getOption("rfacebookstat.api_version"),
			  username          = getOption("rfacebookstat.username"),
              token_path        = fbTokenPath(),
              access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{business_id}{ID by your Business Manager}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your API facebook token}
}

\value{Data frame with project list}

\author{Alexey Seleznev}

\examples{
\dontrun{
bc <- fbGetCatalogs(business_id = 1111111111,
                    access_token = "xxxxxxxx")
 }
}

\name{fbGetBusinessManagers}
\alias{fbGetBusinessManagers}
\title{Get avable business Managers.}
\description{fbGetBusinessManagers get data frame with avable bussines managers list in R.}
\usage{
fbGetBusinessManagers(api_version = getOption("rfacebookstat.api_version"), 
                      username     = getOption("rfacebookstat.username"),
                      token_path   = fbTokenPath(),
                      access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{access_token}{Your facebook API token.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
BM <- fbGetBusinessManagers(api_version = "v2.10", access_token = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
}
}

\name{fbGetLongTimeToken}
\alias{fbGetLongTimeToken}

\title{Get API facebook long time token.}
\description{Change your short time token to long time token.}
\usage{
fbGetLongTimeToken(client_id= NULL,client_secret = NULL,fb_exchange_token = NULL)
}

\arguments{
  \item{client_id}{ Your App ID.}
  \item{client_secret}{Your App secret.}
  \item{fb_exchange_token}{Your short time API token.}
}
\value{API token}

\author{Alexey Seleznev}

\name{fbGetAdAccountUsers}
\alias{fbGetAdAccountUsers}
\title{Get User List From Account}
\description{fbGetAdAccountUsers get data frame with adaccounts user list in R.}
\usage{
fbGetAdAccountUsers(
    accounts_id  = getOption("rfacebookstat.accounts_id"), 
	business_id  = getOption("rfacebookstat.business_id"),
    api_version  = getOption("rfacebookstat.api_version"), 
	console_type = "progressbar", 
	username     = getOption("rfacebookstat.username"),
	token_path   = fbTokenPath(),
	access_token = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{accounts_id}{vector with ID of your ad account, for example c("act_00001","act_00002").}
  \item{business_id}{ID by your Business Manager}
  \item{access_token}{Your facebook API token.}
  \item{console_type}{Console output type, "progressbar" or "message"}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
accountUsers <- fbGetAdAccountUsers(accounts_id = c("act_00001","act_00002"), 
                                    bussiness_id = 1111111,
                                    access_token = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
}
}

\name{fbGetBusinessUserAdAccounts}
\alias{fbGetBusinessUserAdAccounts}
\title{Get a list of accounts for a business manager user}
\description{Loading account list for business manager user.}
\usage{
fbGetBusinessUserAdAccounts(
     business_users_id = NULL, 
     business_id       = getOption("rfacebookstat.business_id"), 
     api_version       = getOption("rfacebookstat.api_version"), 
	 username          = getOption("rfacebookstat.username"),
     token_path        = fbTokenPath(),
     access_token      = getOption("rfacebookstat.access_token"))
}

\arguments{
  \item{business_users_id}{ID by business manager user.}
  \item{business_id}{ID by your Business Manager}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
  \item{api_version}{Current Facebook API version.}
}

\value{
Data frame with ad account list.
}

\author{Alexey Seleznev}

\examples{
\dontrun{
options(rfacebookstat.business_id  = 000000000000,
        rfacebookstat.access_token = "abcde12345....")

my_accs <- fbGetBusinessUserAdAccounts(
                     business_users_id = 777423445991332
					 )
}
}

\name{fbGetAdCreative}
\alias{fbGetAdCreative}

\title{Get creative list from facebook marketing API}
\description{Get ads creatives list with parameters from facebook marketing API. Format which provides layout and contains content for the ad. }
\usage{
fbGetAdCreative(
        accounts_id = getOption("rfacebookstat.accounts_id"), 
        filtering   = NULL,
        api_version = getOption("rfacebookstat.api_version"), 
				username    = getOption("rfacebookstat.username"),
	      token_path  = fbTokenPath(),
				access_token = getOption("rfacebookstat.access_token"),
				limit        = 50)
}
\arguments{
  \item{accounts_id}{Your account ID, for example 'act_11111111111111111'.}
  \item{filtering}{filter creatives}
  \item{api_version}{Current Facebook API version.}
  \item{username}{your username on Facebook}
  \item{token_path}{path to dir with credentials}
  \item{access_token}{Your facebook API token.}
  \item{limit}{Number of rows per API call}
}

\value{
Data frame with ads list.
\item{1. id}{The ID of this creative.}
\item{2. name}{The name of the creative in the creative library.}
\item{3. status}{The status of this creative.}
\item{4. url_tags}{A set of query string parameters which will replace or be appended to urls clicked from page post ads, message of the post, and canvas app install creatives only.}
\item{5. account_id}{The ID of the ad account that this ad belongs to.}
\item{5. page_id}{ID of a Facebook page. An unpublished page post will be created on this page. User must have Admin or Editor role for this page.}
\item{5. link}{Link url.}
\item{6. message}{The main body of the post.}
\item{7. caption}{Link caption.}
\item{8. attachment_style}{The style of the attachment}
\item{9. description}{Link description. }
\item{10. image_hash}{Hash of an image in your image library with Facebook. }
}

\author{Alexey Seleznev}


\examples{
\dontrun{
library(rfacebookstat)
fb_token <- fbGetToken(app_id = 0000000)
ads_creative <- fbGetAdCreative(accounts_id  = "act_11111111111111111",
                                api_version  = 'v3.0',
                                access_token = fb_token)
}

}
